create table sscShippingMethod (
	shippingMethodID number(10,0) not null, 
	class varchar2(255) DEFAULT 'com.softslate.commerce.businessobjects.shipping.ShippingMethodBean', 
	code varchar2(100) not null unique, 
	name varchar2(255), 
	minimum double precision, 
	base double precision, 
	methodType varchar2(255), 
	isActive number(1,0) not null, 
    numberDaysToShip number(10,0),
    methodOrder number(10,0),
	primary key (shippingMethodID)
);

CREATE INDEX IDX_sscShippingMethod_name ON sscShippingMethod (name);
CREATE INDEX IDX_sscSM_methodOrder ON sscShippingMethod (methodOrder);

create table sscShippingRate (
	shippingRateID number(10,0) not null, 
	class varchar2(255) DEFAULT 'com.softslate.commerce.businessobjects.shipping.ShippingRateBean', 
	shippingMethodID number(10,0), 
	floor double precision, 
	ceiling double precision, 
	amount double precision, 
	isActive number(1,0) not null, 
	primary key (shippingRateID)
);

alter table sscShippingRate add constraint FKF20A6BCE3B549038 foreign key (shippingMethodID) references sscShippingMethod;

CREATE INDEX IDX_sscShippingRate_amount ON sscShippingRate (amount);



create table sscShippingRule (
	shippingRuleID number(10,0) not null, 
	class varchar2(255) DEFAULT 'com.softslate.commerce.businessobjects.shipping.ShippingRuleBean', 
	code varchar2(100) not null unique, 
	name varchar2(255), 
	description clob, 
	isActive number(1,0) not null, 
	ruleType varchar2(32), 
	ruleOrder number(10,0) default 0, 
	extra1 varchar2(255), 
	extra2 varchar2(255), 
	extra3 varchar2(255), 
	primary key (shippingRuleID)
);

CREATE INDEX IDX_sscShippingRule_name ON sscShippingRule (name);
CREATE INDEX IDX_sscShippingRule_ruleOrder ON sscShippingRule (ruleOrder);

create table sscSKUShippingRule (
	skuShippingRuleID number(10,0) not null, 
	class varchar(255) DEFAULT 'com.softslate.commerce.businessobjects.shipping.SKUShippingRuleBean', 
	shippingRuleID number(10,0), 
	skuID number(10,0), 
	primary key (skuShippingRuleID)
);

alter table sscSKUShippingRule add constraint FK71A9636769AC6E foreign key (shippingRuleID) references sscShippingRule;
alter table sscSKUShippingRule add constraint FK71A96367E324C735 foreign key (skuID) references sscSKU;


create table sscShippingRuleRange (
	shippingRuleRangeID number(10,0) not null, 
	class varchar(255) DEFAULT 'com.softslate.commerce.businessobjects.shipping.ShippingRuleRangeBean', 
	shippingRuleID number(10,0), 
	floor double precision not null, 
	ceiling double precision not null, 
	isActive number(1,0) not null, 
	discountType varchar2(32), 
	discount double precision, 
	pkgDescription clob, 
	pkgWeight double precision not null, 
	pkgLength double precision not null, 
	pkgHeight double precision not null, 
	pkgWidth double precision not null, 
	pkgExtra1 varchar2(255), 
	pkgExtra2 varchar2(255), 
	pkgExtra3 varchar2(255), 
	pkgExtra4 varchar2(255), 
	pkgExtra5 varchar2(255), 
	countries clob,
	states clob,
	limitMethods clob, 
	limitType varchar2(32), 
	primary key (shippingRuleRangeID)
);
alter table sscShippingRuleRange add constraint FK355DB57369AC6E foreign key (shippingRuleID) references sscShippingRule;


create table sscOrderShippingRule (
	orderShippingRuleID number(10,0) not null, 
	class varchar(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderShippingRuleBean', 
	orderID number(10,0), 
	orderItemID number(10,0), 
	total double precision, 
	shippingRuleID number(10,0), 
	code varchar2(100) not null, 
	name varchar2(255), 
	ruleType varchar2(32), 
	floor double precision, 
	ceiling double precision, 
	discountType varchar2(32), 
	discount double precision, 
	pkgDescription clob, 
	pkgWeight double precision not null, 
	pkgLength double precision not null, 
	pkgHeight double precision not null, 
	pkgWidth double precision not null, 
	limitMethods clob, 
	limitType varchar2(32), 
	extra1 varchar2(255), 
	extra2 varchar2(255), 
	extra3 varchar2(255), 
	pkgExtra1 varchar2(255), 
	pkgExtra2 varchar2(255), 
	pkgExtra3 varchar2(255), 
	pkgExtra4 varchar2(255), 
	pkgExtra5 varchar2(255), 
	countries clob,
	states clob,
	subtotal double precision, 
	primary key (orderShippingRuleID)
);
alter table sscOrderShippingRule add constraint FKB5F9F818DA8BA36 foreign key (orderID) references sscOrder;

CREATE INDEX IDX_sscOShipRule_orderItemID ON sscOrderShippingRule (orderItemID);
CREATE INDEX IDX_sscOrderShippingRule_total ON sscOrderShippingRule (total);
CREATE INDEX IDX_sscOrderShippingRule_code ON sscOrderShippingRule (code);
CREATE INDEX IDX_sscOrderShippingRule_name ON sscOrderShippingRule (name);
