package com.softslate.commerce.administrator.administrator;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.administrator.Administrator;
import com.softslate.commerce.businessobjects.administrator.AdministratorProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/AdministratorEditForm.do". Loads in the current values of the
 * record being edited and populates a <code>AdministratorAddEditForm</code>
 * object.
 * 
 * @author David Tobey
 * 
 */
public class AdministratorEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(AdministratorEditFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		AdministratorAddEditForm baseForm = (AdministratorAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		AdministratorProcessor processor = (AdministratorProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"administratorProcessorImplementer");
		Administrator administrator = processor
				.loadAdministratorFromID(PropertyUtils.describe(form));
		BeanUtils.copyProperties(baseForm, administrator);

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("administratorEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "Administrators",
				"Administrator Details" });
		request.setAttribute("breadcrumbs", bcs);

		request.setAttribute("administrator", administrator);
		request.setAttribute("administratorAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
