package com.softslate.commerce.administrator.order;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderItem;
import com.softslate.commerce.businessobjects.order.OrderItemAttributeProcessor;
import com.softslate.commerce.businessobjects.order.OrderItemProcessor;
import com.softslate.commerce.businessobjects.order.OrderProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to the main administrator view panel for
 * order item attributes. Used for requests to
 * "/administrator/OrderItemAttribute.do".
 * <p>
 * The <code>execute</code> method of this class populates a
 * <code>Collection</code> named <code>itemCollection</code> in the request
 * scope containing the items to be displayed. Next, it sets a request scope
 * variable named <code>editPermission</code> that determines if the current
 * user has permission to edit the items. Finally, it creates a
 * <code>Collection</code> named <code>breadcrumbs</code> containing the
 * links and labels of each breadcrumb for display.
 * 
 * @author David Tobey
 * 
 */
public class OrderItemAttributeAction extends BaseAction {

	static Log log = LogFactory.getLog(OrderItemAttributeAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		
		OrderItemAttributeForm baseForm = (OrderItemAttributeForm) form;
		baseForm.setProperties(request);
		request.setAttribute("mainForm", baseForm);

		// Get the collection of items to display
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		OrderItemAttributeProcessor processor =  bof
				.createObject(OrderItemAttributeProcessor.class);
		Map orderItemAttributesAndCount = processor
				.loadOrderItemAttributesAndCount(PropertyUtils
						.describe(baseForm));
		request.setAttribute("orderItemAttributes", orderItemAttributesAndCount
				.get("items"));
		request.setAttribute("itemCollection", orderItemAttributesAndCount
				.get("items"));
		request.setAttribute("itemCount", orderItemAttributesAndCount
				.get("itemCount"));
		request.setAttribute("assignedItems", orderItemAttributesAndCount
				.get("assignedItems"));

		// Load the parent order item
		OrderItem orderItem = null;
		if (baseForm.getOrderItemID() != null
				&& baseForm.getOrderItemID().length() > 0) {
			OrderItemProcessor parentProcessor =  bof
					.createObject(OrderItemProcessor.class);
			orderItem = parentProcessor.loadOrderItemFromID(PropertyUtils
					.describe(baseForm));
			request.setAttribute("orderItem", orderItem);
		}

		// Load the parent order if an id is present
		Order order = null;
		if (orderItem != null && orderItem.getOrderID() > 0) {
			Map parameters = new HashMap();
			parameters.put("orderID", Integer.toString(orderItem.getOrderID()));
			OrderProcessor parentProcessor =  bof
					.createObject(OrderProcessor.class);
			order = parentProcessor.loadOrderFromID(parameters);
			request.setAttribute("order", order);
		}

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("orderItemAttributeEdit")) {
			request.setAttribute("editPermission", "false");
			baseForm.setEditMode("false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "Orders",
				"Order Details", "Order Items", "Order Item Details",
				"Order Item Attributes" });
		request.setAttribute("breadcrumbs", bcs);

		// Create childSubNavigation collection
		Collection childsn = Arrays.asList(new String[] { "Order Item Details",
				"Order Item Attributes", "Add Order Item Attribute" });
		request.setAttribute("childSubNavigation", childsn);

		// Forward according to the outputMode of the request
		if (baseForm.getOutputMode().equals("print")) {
			request.setAttribute("outputMode", "print");
			baseForm.setOutputMode("screen");
			return mapping.findForward("successSimpleLayout");
		} else if (baseForm.getOutputMode().equals("excel")) {
			request.setAttribute("outputMode", "excel");
			baseForm.setOutputMode("screen");
			response.setHeader("Content-Type", "application/vnd.ms-excel");
			response.setHeader("Content-Disposition",
					"attachment; filename=orderItemAttributes.xls");
			return mapping.findForward("successSimpleLayout");
		}
		return mapping.findForward("success");
	}
}
