package com.softslate.commerce.administrator.product;

import java.util.Collection;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.Category;
import com.softslate.commerce.businessobjects.product.CategoryProcessor;
import com.softslate.commerce.businessobjects.product.Attribute;
import com.softslate.commerce.businessobjects.product.AttributeProcessor;
import com.softslate.commerce.businessobjects.product.Manufacturer;
import com.softslate.commerce.businessobjects.product.ManufacturerProcessor;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.businessobjects.customer.CustomerWishList;
import com.softslate.commerce.businessobjects.customer.CustomerWishListProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to the main administrator view panel for
 * products. Used for requests to "/administrator/Product.do".
 * <p>
 * The <code>execute</code> method of this class populates a
 * <code>Collection</code> named <code>itemCollection</code> in the request
 * scope containing the items to be displayed. Next, it sets a request scope
 * variable named <code>editPermission</code> that determines if the current
 * user has permission to edit the items. Finally, it creates a
 * <code>Collection</code> named <code>breadcrumbs</code> containing the
 * links and labels of each breadcrumb for display.
 * 
 * @author David Tobey
 * 
 */
public class ProductAction extends BaseAction {

	static Log log = LogFactory.getLog(ProductAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		// Handle the main form for the control screen
		ProductForm baseForm = (ProductForm) form;
		baseForm.setProperties(request);
		request.setAttribute("mainForm", baseForm);

		// Handle the quick-add form
		ProductAddEditForm addForm = (ProductAddEditForm) request
				.getAttribute("productAddEditForm");
		if (addForm == null
				|| (request.getAttribute("reset") != null && request
						.getAttribute("reset").equals("true"))) {
			addForm = new ProductAddEditForm();
			if (baseForm.getCategoryID().length() > 0) {
				addForm.setPrimaryCategoryID(baseForm.getCategoryID());
			}
		}
		addForm.initializeProperties(mapping, request);
		request.setAttribute("productAddEditForm", addForm);

		// Get the collection of items to display
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		ProductProcessor processor =  bof
				.createObject(ProductProcessor.class);
		Map productsAndCount = processor.loadProductsAndCount(PropertyUtils
				.describe(baseForm));
		request.setAttribute("products", productsAndCount.get("items"));
		request.setAttribute("itemCollection", productsAndCount.get("items"));
		request.setAttribute("itemCount", productsAndCount.get("itemCount"));
		request.setAttribute("assignedItems", productsAndCount
				.get("assignedItems"));

		// Bug where the primaryCategoryID field gets renamed to
		// primaryCategory
		for (int j = 0; j < baseForm.getDisplayFields().length; j++) {
			String displayField = baseForm.getDisplayFields()[j];
			if (displayField.equals("primaryCategory")) {
				displayField = "primaryCategoryID";
				baseForm.getDisplayFields()[j] = "primaryCategoryID";
			}
		}

		// Load all categories for the primary category drop down.
		CategoryProcessor catProcessor =  bof
				.createObject(CategoryProcessor.class);
		Collection categories = catProcessor.loadAllCategories();
		request.setAttribute("categories", categories);

		// Load all manufacturers for the manufacturer drop down.
		if (baseForm.getEditMode() != null
				&& baseForm.getEditMode().equals("true")
				&& Arrays.asList(baseForm.getDisplayFields()).contains(
						"manufacturerID")) {
			ManufacturerProcessor manfProcessor =  bof
					.createObject(ManufacturerProcessor.class);
			Collection manufacturers = manfProcessor.loadAllManufacturers();
			request.setAttribute("manufacturerCollection", manufacturers);
		}

		// Load the parent if an id is present
		Category category = null;
		if (baseForm.getCategoryID() != null
				&& baseForm.getCategoryID().length() > 0) {
			CategoryProcessor categoryProcessor =  bof
					.createObject(CategoryProcessor.class);
			category = categoryProcessor.loadCategoryFromID(PropertyUtils
					.describe(baseForm));
			request.setAttribute("category", category);
		}
		Manufacturer manufacturer = null;
		if (baseForm.getParentManufacturerID() != null
				&& baseForm.getParentManufacturerID().length() > 0) {
			ManufacturerProcessor manufacturerProcessor =  bof
					.createObject(ManufacturerProcessor.class);
			Map parameters = new HashMap();
			parameters
					.put("manufacturerID", baseForm.getParentManufacturerID());
			manufacturer = manufacturerProcessor
					.loadManufacturerFromID(parameters);
			request.setAttribute("manufacturer", manufacturer);
		}
		Product product = null;
		if (baseForm.getParentProductID() != null
				&& baseForm.getParentProductID().length() > 0) {
			ProductProcessor productProcessor =  bof
					.createObject(ProductProcessor.class);
			Map parameters = new HashMap();
			parameters.put("productID", baseForm.getParentProductID());
			product = productProcessor.loadProductFromID(parameters);
			request.setAttribute("product", product);
		}
		Attribute attribute = null;
		if (baseForm.getAttributeID() != null
				&& baseForm.getAttributeID().length() > 0) {
			AttributeProcessor attributeProcessor =  bof
					.createObject(AttributeProcessor.class);
			attribute = attributeProcessor.loadAttributeFromID(PropertyUtils
					.describe(baseForm));
			request.setAttribute("attribute", attribute);
		}
		CustomerWishList customerWishList = null;
		if (baseForm.getCustomerWishListID() != null
				&& baseForm.getCustomerWishListID().length() > 0) {
			CustomerWishListProcessor customerWishListProcessor =  bof
					.createObject(CustomerWishListProcessor.class);
			customerWishList = customerWishListProcessor.loadCustomerWishListFromID(PropertyUtils
					.describe(baseForm));
			request.setAttribute("customerWishList", customerWishList);
			// Set the customer to help with the navigation
			if (customerWishList != null && customerWishList.getCustomer() != null){
				request.setAttribute("customer", customerWishList.getCustomer());
			}
		}

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("productEdit")) {
			request.setAttribute("editPermission", "false");
			baseForm.setEditMode("false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Navigation
		if (baseForm.getCategoryID() != null
				&& baseForm.getCategoryID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Categories",
					"Category Details", "Products" });
			if (category.getIsContentPage()) {
				bcs = Arrays.asList(new String[] { "Home", "Content Pages",
						"Content Page Details", "Products" });
			}
			request.setAttribute("breadcrumbs", bcs);

		} else if (baseForm.getParentProductID() != null
				&& baseForm.getParentProductID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Products",
					"Product Details", "Related Products" });
			request.setAttribute("breadcrumbs", bcs);

		} else if (baseForm.getParentManufacturerID() != null
				&& baseForm.getParentManufacturerID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home",
					"Manufacturers", "Manufacturer Details", "Products" });
			request.setAttribute("breadcrumbs", bcs);

		} else if (baseForm.getAttributeID() != null
				&& baseForm.getAttributeID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Attributes",
					"Attribute Details", "Products" });
			request.setAttribute("breadcrumbs", bcs);

		} else if (baseForm.getCustomerWishListID() != null
				&& baseForm.getCustomerWishListID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Customer Wish Lists",
					"Customer Wish List Details", "Products" });
			request.setAttribute("breadcrumbs", bcs);

		} else {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Products" });
			request.setAttribute("breadcrumbs", bcs);
		}

		// Forward according to the outputMode of the request
		if (baseForm.getOutputMode().equals("print")) {
			request.setAttribute("outputMode", "print");
			baseForm.setOutputMode("screen");
			return mapping.findForward("successSimpleLayout");
		} else if (baseForm.getOutputMode().equals("excel")) {
			request.setAttribute("outputMode", "excel");
			baseForm.setOutputMode("screen");
			response.setHeader("Content-Type", "application/vnd.ms-excel");
			response.setHeader("Content-Disposition",
					"attachment; filename=products.xls");
			return mapping.findForward("successSimpleLayout");
		}
		return mapping.findForward("success");
	}
}
