package com.softslate.commerce.administrator.product;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.UUID;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.ProductSettingProcessor;
import com.softslate.commerce.businessobjects.storage.FileStorageProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseAction;
import com.thebuzzmedia.imgscalr.Scalr;

/**
 * Adds a setting to the system, but also uploads an image and resizes it for
 * display.
 * 
 * @author Tony Zakula
 */
public class ProductSettingAddImageAction extends BaseAction {

	static Log log = LogFactory.getLog(ProductSettingAddImageAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting Image Upload action.");

		ProductSettingAddEditFormImages baseForm = (ProductSettingAddEditFormImages) form;
		baseForm.initializeProperties(mapping, request);
		// Upload images and resize according to settings.
		if (baseForm.getAdditionalImageUpload() != null
				&& baseForm.getAdditionalImageUpload().getFileSize() > 0) {

			ActionUtils actionUtils = (ActionUtils) baseForm
					.getBusinessObjectFactory().createUtilsObject(
							"actionUtilsImplementer");
			FileStorageProcessor fsp = (FileStorageProcessor) baseForm.getBusinessObjectFactory()
					.createObject("fileStorageProcessorImplementer");
			ProductSettingProcessor processor = (ProductSettingProcessor) baseForm.getBusinessObjectFactory()
					.createObject("productSettingProcessorImplementer");

			String smallImageSize = baseForm.getSettings()
					.getValue("smallImageSize").toString();
			String mediumImageSize = baseForm.getSettings()
					.getValue("mediumImageSize").toString();
			String largeImageSize = baseForm.getSettings()
					.getValue("largeImageSize").toString();

			// Load the imageSystemPath and imagePathURL settings; derive the
			// imageSystemPath from the URL path if blank.
			String imageSystemPath = baseForm.getSettings().getValue("imageSystemPath").toString();
			String imagePathURL = baseForm.getSettings().getValue("imagePathURL").toString();
			if (!imagePathURL.endsWith("/")) {
				imagePathURL = imagePathURL + "/";
			}
			if (imageSystemPath == null || imageSystemPath.length() == 0) {
				imageSystemPath = getServlet().getServletContext().getRealPath(imagePathURL)
						+ System.getProperty("file.separator");
			}
			if (!imageSystemPath.endsWith(System.getProperty("file.separator"))) {
				imageSystemPath = imageSystemPath + System.getProperty("file.separator");
			}
			imagePathURL = baseForm.getRequest().getContextPath() + imagePathURL;
			String filename = actionUtils.makeFilenameNice(baseForm.getAdditionalImageUpload().getFileName());

			// Scale and save and insert the large image
			InputStream largeIS = null;
			InputStream smallIS = null;
			InputStream mediumIS = null;
			try {

				// Scale save and insert large image.
				largeIS = resizeImage(baseForm.getAdditionalImageUpload().getInputStream(),
						Integer.parseInt(largeImageSize));
				String largefilename = "LG-" + filename;
				fsp.storeFile(largeIS, imageSystemPath + largefilename, "largeAdditionalProduct");

				// Set the form properties
				baseForm.setCode(UUID.randomUUID().toString());
				baseForm.setName("LARGE");
				baseForm.setDescription(fsp.urlForFile(imagePathURL, largefilename, "largeAdditionalProduct"));
				processor.addProductSetting(PropertyUtils.describe(form));

				// Scale save and insert medium image.
				mediumIS = resizeImage(baseForm.getAdditionalImageUpload().getInputStream(),
						Integer.parseInt(mediumImageSize));
				String mediumfilename = "MD-" + filename;
				fsp.storeFile(mediumIS, imageSystemPath + mediumfilename, "mediumAdditionalProduct");

				// Set the form properties
				baseForm.setCode(UUID.randomUUID().toString());
				baseForm.setName("MEDIUM");
				baseForm.setDescription(fsp.urlForFile(imagePathURL, mediumfilename, "mediumAdditionalProduct"));
				processor.addProductSetting(PropertyUtils.describe(form));

				// Scale save and insert small image.
				smallIS = resizeImage(baseForm.getAdditionalImageUpload().getInputStream(),
						Integer.parseInt(smallImageSize));
				String smallfilename = "SM-" + filename;
				fsp.storeFile(smallIS, imageSystemPath + smallfilename, "smallAdditionalProduct");

				// Set the form properties
				baseForm.setCode(UUID.randomUUID().toString());
				baseForm.setName("SMALL");
				baseForm.setDescription(fsp.urlForFile(imagePathURL, smallfilename, "smallAdditionalProduct"));
				processor.addProductSetting(PropertyUtils.describe(form));
			} finally {
				if (largeIS != null)
					largeIS.close();
				if (mediumIS != null)
					mediumIS.close();
				if (smallIS != null)
					smallIS.close();
			}
		}

		// Set the success message
		baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
				new ActionMessage("messages.recordAdded"));
		saveMessages(baseForm.getRequest(), baseForm.getMessages());

		// Reset the form
		Class classDefinition = baseForm.getClass();
		baseForm = (ProductSettingAddEditFormImages) classDefinition
				.newInstance();
		baseForm.initializeProperties(mapping, request);
		request.setAttribute("productSettingAddEditForm", baseForm);

		return mapping.findForward("success");
	}

	private InputStream resizeImage(InputStream inputStream, int imageSize) throws IOException {
		BufferedImage srcImage = ImageIO.read(inputStream);
		BufferedImage scaledImage = Scalr.resize(srcImage, imageSize);
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		ImageIO.write(scaledImage, "jpg", os);
		byte[] scaledImageContents = os.toByteArray();
		InputStream is = new ByteArrayInputStream(scaledImageContents);
		os.close();
		return is;
	}

}
