package com.softslate.commerce.administrator.shipping;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.BaseDynaForm;

/**
 * @author Jason McWilliams
 * 
 */
public class SettingsFormUPS extends BaseDynaForm {

	private static final long serialVersionUID = -3790457312228631672L;

	static Log log = LogFactory.getLog(SettingsFormUPS.class);

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isDebugEnabled())
			log.debug("Starting validation.");
		initializeProperties(mapping, request);

		// Required
		if (request.getParameter("UPS_Url") == null
				|| request.getParameter("UPS_Url").length() == 0) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", "UPS URL"));
		}

		// Required
		if (request.getParameter("UPS_accessLicenseNumber") == null
				|| request.getParameter("UPS_accessLicenseNumber").length() == 0) {
			getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required",
							"Access License Number"));
		}

		// Required
		if (request.getParameter("UPS_userId") == null
				|| request.getParameter("UPS_userId").length() == 0) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", "UPS User ID"));
		}

		// Required
		if (request.getParameter("UPS_password") == null
				|| request.getParameter("UPS_password").length() == 0) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", "UPS Password"));
		}

		// Required
		if (request.getParameter("UPS_pickupTypeCode") == null
				|| request.getParameter("UPS_pickupTypeCode").length() == 0) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", "Pickup Type Code"));
		}

		// Required
		if (request.getParameter("UPS_customerClassificationCode") == null
				|| request.getParameter("UPS_customerClassificationCode")
						.length() == 0) {
			getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required",
							"Customer Classification Code"));
		}

		// Required
		if (!getFormUtils().stringArrayHasAnElement(this,
				"UPS_validServiceCodesArray")) {
			getErrors()
					.add(
							ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.required",
									"Valid Service Codes"));
		} else {
			String[] codes = (String[]) this.get("UPS_validServiceCodesArray");
			String codeString = "";
			for (int i = 0; i < codes.length; i++) {
				if (codeString.length() > 0) {
					codeString = codeString + ",";
				}
				codeString = codeString + codes[i];
			}
			this.set("UPS_validServiceCodes", codeString);
		}

		if (request.getParameter("UPS_shipperNumber") == null
				|| request.getParameter("UPS_shipperNumber").length() == 0) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", "Shipper Number"));
		}

		// Required
		if (request.getParameter("UPS_shipperCountryCode") == null
				|| request.getParameter("UPS_shipperCountryCode").length() == 0) {
			getErrors()
					.add(
							ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.required",
									"Shipper Country Code"));
		}

		if (request.getParameter("UPS_residentialAddressIndicator") == null
				|| request.getParameter("UPS_residentialAddressIndicator")
						.length() == 0) {
			getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required",
							"Residential Address Indicator"));
		}

		if (request.getParameter("UPS_shipFromCountryCode") == null
				|| request.getParameter("UPS_shipFromCountryCode").length() == 0) {
			getErrors()
					.add(
							ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.required",
									"Senders Country Code"));
		}

		// Required
		if (request.getParameter("UPS_unitOfMeasurementCode") == null
				|| request.getParameter("UPS_unitOfMeasurementCode").length() == 0) {
			getErrors()
					.add(
							ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.required",
									"Unit of Measurement"));
		}
		if (request.getParameter("UPS_packageCodeType") == null
				|| request.getParameter("UPS_packageCodeType").length() == 0) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", "Package Code Type"));
		}
		if (request.getParameter("UPS_useInsuredValue") == null
				|| request.getParameter("UPS_useInsuredValue").length() == 0) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", "Use Insured Value"));
		}
		if (request.getParameter("UPS_insuredValueCurrencyCode") == null
				|| request.getParameter("UPS_insuredValueCurrencyCode")
						.length() == 0) {
			getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required",
							"Insured Value Currency Code"));
		}
		if (request.getParameter("UPS_rateInformationNegotiatedRatesIndicator") == null
				|| request.getParameter(
						"UPS_rateInformationNegotiatedRatesIndicator").length() == 0) {
			getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required",
							"Negotiated Rates Indicator"));
		}
		if (request.getParameter("UPS_packageUseDimensions") == null
				|| request.getParameter("UPS_packageUseDimensions").length() == 0) {
			getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required",
							"Use Package Dimensions"));
		}

		if (request.getParameter("UPS_packageDimensionsUomCode") == null
				|| request.getParameter("UPS_packageDimensionsUomCode")
						.length() == 0) {
			getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required",
							"Unit of Measurement for package dimensions."));
		}

		return getErrors();
	}

}
