package com.softslate.commerce.businessobjects.email;

import java.io.StringWriter;
import java.io.Writer;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.tools.generic.ComparisonDateTool;
import org.apache.velocity.tools.generic.DateTool;
import org.apache.velocity.tools.generic.NumberTool;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;

public class TemplateParserImpl extends BaseBusinessProcessor implements TemplateParser {

	static Log log = LogFactory.getLog(TemplateParserImpl.class);

	private Template template = null;
	private Email email = null;
	private Map<String, Object> extraTemplateObjects = null;

	public void initialize(Template template, Email email, Map<String, Object> extraTemplateObjects) {
		this.template = template;
		this.email = email;
		this.extraTemplateObjects = extraTemplateObjects;
	}

	public Template getTemplate() {
		return template;
	}

	public void setTemplate(Template template) {
		this.template = template;
	}

	public Email getEmail() {
		return email;
	}

	public void setEmail(Email email) {
		this.email = email;
	}

	public Map<String, Object> getExtraTemplateObjects() {
		return extraTemplateObjects;
	}

	public void setExtraTemplateObjects(Map<String, Object> extraTemplateObjects) {
		this.extraTemplateObjects = extraTemplateObjects;
	}

	public boolean parseEmail() {
		Properties p = new Properties();
		p.setProperty("runtime.log.logsystem.class", "org.apache.velocity.runtime.log.NullLogSystem");
		Velocity.init(p);
		Writer bodyHtmlOut = new StringWriter();
		Writer bodyTextOut = new StringWriter();
		Writer subjectOut = new StringWriter();
		try {
			VelocityContext context1 = new VelocityContext();
			context1.put("order", getEmail().getOrder());
			context1.put("customer", getEmail().getCustomer());
			context1.put("settings", getSettings());
			context1.put("StringUtils", StringUtils.class);
			context1.put("date", new DateTool());
			context1.put("comparisonDate", new ComparisonDateTool());
			context1.put("number", new NumberTool());
			if (getExtraTemplateObjects() != null && getExtraTemplateObjects().size() > 0) {
				Iterator<Entry<String, Object>> it = getExtraTemplateObjects().entrySet().iterator();
				while (it.hasNext()) {
					Map.Entry<String, Object> pairs = (Map.Entry<String, Object>) it.next();
					context1.put(pairs.getKey(), pairs.getValue());
				}
			}
			if (StringUtils.isNotBlank(getTemplate().getBodyHtml())) {
				Velocity.evaluate(context1, bodyHtmlOut, getTemplate().getCode(), getTemplate().getBodyHtml());
				getEmail().setBodyHtml(bodyHtmlOut.toString());
			}
			if (StringUtils.isNotBlank(getTemplate().getBodyText())) {
				Velocity.evaluate(context1, bodyTextOut, getTemplate().getCode(), getTemplate().getBodyText());
				getEmail().setBodyText(bodyTextOut.toString());
			}
			if (StringUtils.isNotBlank(getTemplate().getSubject())) {
				Velocity.evaluate(context1, subjectOut, getTemplate().getCode(), getTemplate().getSubject());
				getEmail().setSubject(subjectOut.toString());
			}
		} catch (Exception ex) {
			if (log.isErrorEnabled()) {
				log.error("Unable to parse template for email! ", ex);
			}
			return false;
		}

		return true;
	}

}
