package com.softslate.commerce.businessobjects.importexport;

import java.beans.PropertyDescriptor;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.businessobjects.core.BusinessObject;
import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.businessobjects.order.OrderProcessor;
import com.softslate.commerce.businessobjects.product.AttributeProcessor;
import com.softslate.commerce.businessobjects.product.CategoryProcessor;
import com.softslate.commerce.businessobjects.product.OptionProcessor;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.businessobjects.product.ProductSettingProcessor;
import com.softslate.commerce.businessobjects.product.SKUProcessor;
import com.softslate.commerce.daos.customer.CustomerGatewayDAO;
import com.softslate.commerce.daos.order.OrderGatewayDAO;
import com.softslate.commerce.daos.product.AttributeDAO;
import com.softslate.commerce.daos.product.AttributeGatewayDAO;
import com.softslate.commerce.daos.product.CategoryDAO;
import com.softslate.commerce.daos.product.CategoryGatewayDAO;
import com.softslate.commerce.daos.product.ManufacturerDAO;
import com.softslate.commerce.daos.product.OptionDAO;
import com.softslate.commerce.daos.product.OptionGatewayDAO;
import com.softslate.commerce.daos.product.ProductDAO;
import com.softslate.commerce.daos.product.ProductGatewayDAO;
import com.softslate.commerce.daos.product.ProductSettingDAO;
import com.softslate.commerce.daos.product.SKUDAO;
import com.softslate.commerce.daos.product.SKUGatewayDAO;

/**
 * Base class with common methods handling the processing of text file imports
 * and exports.
 * 
 * @author David Tobey, Ashish Jain
 */
public class BasicImportExportProcessor extends BaseBusinessProcessor implements ImportExportProcessor {

	static Log log = LogFactory.getLog(BasicImportExportProcessor.class);

	public final static String PRODUCT_TABLE = "sscproduct";

	public final static String CATEGORY_TABLE = "ssccategory";

	public final static String ATTRIBUTE_TABLE = "sscattribute";

	public final static String SKU_TABLE = "sscsku";

	public final static String OPTION_TABLE = "sscoption";

	public final static String ORDER_TABLE = "sscorder";

	public final static String SHIPPINGRULE_TABLE = "sscshippingrule";

	public final static String SHIPPINGRULERANGE_TABLE = "sscshippingrulerange";

	public final static String CUSTOMER_TABLE = "ssccustomer";

	public final static String PRODUCT_SETTING_TABLE = "sscproductsetting";

	public ProductDAO productDAO = null;

	public CategoryDAO categoryDAO = null;

	public AttributeDAO attributeDAO = null;

	public OptionDAO optionDAO = null;

	public ManufacturerDAO manufacturerDAO = null;

	public SKUDAO skuDAO = null;

	public ProductSettingDAO productSettingDAO = null;

	public Collection loadColumnNames(String tableName, boolean filterId) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading column names for " + tableName);

		Collection columns = new ArrayList();

		String type = this.getImplementorNameFromTableName(tableName);

		String className = getAppComponents().getProperty(type);
		Class classDefinition = Class.forName(className);
		BusinessObject businessObject = (BusinessObject) classDefinition.newInstance();
		PropertyDescriptor descriptors[] = PropertyUtils.getPropertyDescriptors(businessObject);
		if (null != descriptors && descriptors.length > 0) {
			columns = new ArrayList();
			for (int i = 0; i < descriptors.length; i++) {

				// Skip if this is an indexed property
				if (descriptors[i].getPropertyType() != null) {
					String pType = descriptors[i].getPropertyType().getName();
					if (!pType.endsWith("Class") && !pType.startsWith("com.softslate.commerce.businessobjects")
							&& !pType.equalsIgnoreCase("java.util.Collection")
							&& !pType.equalsIgnoreCase("java.util.Map")) {
						columns.add(descriptors[i].getName());
					}
				}
			}
			if (tableName.equalsIgnoreCase(OPTION_TABLE)) {
				columns.add("attributeCode");
			}
			if (tableName.equalsIgnoreCase(PRODUCT_TABLE)) {
				columns.add("primaryCategoryCode");
				columns.add("manufacturerCode");
				columns.add("category1");
				columns.add("category2");
				columns.add("category3");
				columns.add("category4");
				columns.add("category5");
			}
			if (tableName.equalsIgnoreCase(CATEGORY_TABLE)) {
				columns.add("parentCategoryCode");
			}
			if (tableName.equalsIgnoreCase(SKU_TABLE)) {
				columns.add("productCode");
			}
			if (tableName.equalsIgnoreCase(SHIPPINGRULERANGE_TABLE)) {
				columns.add("shippingRuleCode");
			}
			if (tableName.equalsIgnoreCase(PRODUCT_SETTING_TABLE)) {
				columns.add("productCode");
			}
			if (filterId)
				columns = this.filterColumns(columns, tableName);
		}
		return columns;
	}

	public String getImplementorNameFromTableName(String tableName) throws Exception {
		if (tableName.equals(PRODUCT_TABLE))
			return "productImplementer";
		else if (tableName.equals(CATEGORY_TABLE))
			return "categoryImplementer";
		else if (tableName.equals(ATTRIBUTE_TABLE))
			return "attributeImplementer";
		else if (tableName.equals(OPTION_TABLE))
			return "optionImplementer";
		else if (tableName.equals(SKU_TABLE))
			return "skuImplementer";
		else if (tableName.equals(PRODUCT_SETTING_TABLE))
			return "productSettingImplementer";
		else if (tableName.equals(ORDER_TABLE))
			return "ordersExportImplementer";
		else if (tableName.equals(SHIPPINGRULE_TABLE))
			return "shippingRuleImplementer";
		else if (tableName.equals(SHIPPINGRULERANGE_TABLE))
			return "shippingRuleRangeImplementer";
		else if (tableName.equals(CUSTOMER_TABLE))
			return "customerImplementer";
		else
			throw new Exception("This table transformation not supported, Tablename : " + tableName);
	}

	public Collection filterColumns(Collection columns, String tableName) throws Exception {
		if (null != tableName) {
			String[] filterColumnNames = null;
			if (tableName.equals(PRODUCT_TABLE))
				filterColumnNames = new String[] { "productid", "primarycategoryid", "manufacturerid" };
			else if (tableName.equals(CATEGORY_TABLE))
				filterColumnNames = new String[] { "categoryid", "categorytreeid", "categoryLevel", "categorySettings",
						"productCategories", "firstRow", "itemCount", "itemsPerPage" };
			else if (tableName.equals(ATTRIBUTE_TABLE))
				filterColumnNames = new String[] { "attributeid", "defaultoptionid" };
			else if (tableName.equals(OPTION_TABLE))
				filterColumnNames = new String[] { "attribute", "optionid", "attributeid" };
			else if (tableName.equals(SKU_TABLE))
				filterColumnNames = new String[] { "skuid", "productid", "attributeid1", "attributeid2",
						"attributeid3", "attributeid4", "attributeid5", "optionid1", "optionid2", "optionid3",
						"optionid4", "optionid5" };
			else if (tableName.equals(PRODUCT_SETTING_TABLE))
				filterColumnNames = new String[] { "productSettingID", "product", "productID", "value" };
			else if (tableName.equals(ORDER_TABLE))
				filterColumnNames = new String[] { "orderid", "customerid" };
			else if (tableName.equals(SHIPPINGRULE_TABLE))
				filterColumnNames = new String[] { "shippingruleid", "shippingRuleRanges" };
			else if (tableName.equals(SHIPPINGRULERANGE_TABLE))
				filterColumnNames = new String[] { "shippingrulerangeid", "shippingruleid", "shippingRule" };
			else if (tableName.equals(CUSTOMER_TABLE))
				filterColumnNames = new String[] { "customerid", "customerAddressCollection", "billingAddress1",
						"billingAddress2", "billingCity", "billingState", "billingCountry", "billingEmail1",
						"billingEmail2", "billingExtra1", "billingExtra2", "billingExtra3", "billingExtra4",
						"billingExtra5", "billingFirstName", "billingLastName", "billingNotes", "billingOrganization",
						"billingPhone1", "billingPhone2", "billingPostalCode" };
			else
				throw new Exception("This table transformation not supported, Tablename : " + tableName);

			return this.filterColumns(columns, filterColumnNames);
		}
		return null;

	}

	public Collection filterColumns(Collection columns, String[] filterColumnNames) {

		if (null != filterColumnNames) {
			Iterator iterator = columns.iterator();
			while (iterator.hasNext()) {
				String columnName = (String) iterator.next();
				if (columnName.equalsIgnoreCase("class"))
					iterator.remove();
				else if (columnName.startsWith("parent") && !columnName.equalsIgnoreCase("parentCategoryCode"))
					iterator.remove();
				else if (columnName.endsWith("Collection"))
					iterator.remove();
				else
					for (int i = 0; i < filterColumnNames.length; i++)
						if (columnName.equalsIgnoreCase(filterColumnNames[i]))
							iterator.remove();
			}
		}
		return columns;

	}

	public int loadMaxID(Map parameters) throws Exception {
		String tableName = (String) parameters.get("tableName");
		if (null != tableName) {
			// String[] filterColumnNames = null;
			if (tableName.equals(PRODUCT_TABLE))
				return this.loadMaxProductID(parameters);
			else if (tableName.equals(CATEGORY_TABLE))
				return this.loadMaxCategoryID(parameters);
			else if (tableName.equals(ATTRIBUTE_TABLE))
				return this.loadMaxAttributeID(parameters);
			else if (tableName.equals(OPTION_TABLE))
				return this.loadMaxOptionID(parameters);
			else if (tableName.equals(SKU_TABLE))
				return this.loadMaxSKUID(parameters);
			else if (tableName.equals(ORDER_TABLE))
				return this.loadMaxOrderID(parameters);
			else if (tableName.equals(CUSTOMER_TABLE))
				return this.loadMaxCustomerID(parameters);
			else if (tableName.equals(PRODUCT_SETTING_TABLE))
				return this.loadMaxProductSettingID(parameters);
			else
				throw new Exception("This table transformation not supported, Tablename : " + tableName);
		}
		return 0;
	}

	public int loadMaxProductID(Map parameters) throws Exception {
		ProductProcessor productProcessor = (ProductProcessor) getBusinessObjectFactory().createObject(
				"productProcessorImplementer");
		return productProcessor.loadMaxProductID();
	}

	public int loadMaxCategoryID(Map parameters) throws Exception {
		CategoryProcessor categoryProcessor = (CategoryProcessor) getBusinessObjectFactory().createObject(
				"categoryProcessorImplementer");
		return categoryProcessor.loadMaxCategoryID();
	}

	public int loadMaxAttributeID(Map parameters) throws Exception {
		AttributeProcessor attributeProcessor = (AttributeProcessor) getBusinessObjectFactory().createObject(
				"attributeProcessorImplementer");
		return attributeProcessor.loadMaxAttributeID();
	}

	public int loadMaxOptionID(Map parameters) throws Exception {
		OptionProcessor optionProcessor = (OptionProcessor) getBusinessObjectFactory().createObject(
				"optionProcessorImplementer");
		return optionProcessor.loadMaxOptionID();
	}

	public int loadMaxSKUID(Map parameters) throws Exception {
		SKUProcessor skuProcessor =  getBusinessObjectFactory().createObject(SKUProcessor.class);
		return skuProcessor.loadMaxSKUID();
	}

	public int loadMaxOrderID(Map parameters) throws Exception {
		OrderProcessor orderProcessor = (OrderProcessor) getBusinessObjectFactory().createObject(
				"orderProcessorImplementer");
		return orderProcessor.loadMaxOrderID();
	}

	public int loadMaxCustomerID(Map parameters) throws Exception {
		CustomerProcessor customerProcessor = (CustomerProcessor) getBusinessObjectFactory().createObject(
				"customerProcessorImplementer");
		return customerProcessor.loadMaxCustomerID();
	}

	public int loadMaxProductSettingID(Map parameters) throws Exception {
		ProductSettingProcessor productSettingProcessor =  getBusinessObjectFactory()
				.createObject(ProductSettingProcessor.class);
		return productSettingProcessor.loadMaxProductSettingID();
	}

	public Collection processExport(Map parameters) throws Exception {

		// Clear the Hibernate session so the exported records do not accumulate
		// in memory
		getDaoFactory().getSession().clear();

		// Call the export method based on the table being exported
		String tableName = (String) parameters.get("tableName");
		if (null != tableName) {
			// String[] filterColumnNames = null;
			if (tableName.equals(PRODUCT_TABLE))
				return this.processProductExport(parameters);
			else if (tableName.equals(CATEGORY_TABLE))
				return this.processCategoryExport(parameters);
			else if (tableName.equals(ATTRIBUTE_TABLE))
				return this.processAttributeExport(parameters);
			else if (tableName.equals(OPTION_TABLE))
				return this.processOptionExport(parameters);
			else if (tableName.equals(SKU_TABLE))
				return this.processSKUExport(parameters);
			else if (tableName.equals(ORDER_TABLE))
				return this.processOrderExport(parameters);
			else if (tableName.equals(CUSTOMER_TABLE))
				return this.processCustomerExport(parameters);
			else if (tableName.equals(PRODUCT_SETTING_TABLE))
				return this.processProductSettingExport(parameters);
			else
				throw new Exception("This table transformation not supported, Tablename : " + tableName);
		}
		return null;
	}

	public Collection processProductExport(Map parameters) throws Exception {
		String categoriesAll = (String) parameters.get("categoriesAll");
		int firstID = ((Integer) parameters.get("firstID")).intValue();
		int lastID = ((Integer) parameters.get("lastID")).intValue();
		Collection products = null;
		ProductGatewayDAO gatewayDAO =  getDaoFactory().createDAO(ProductGatewayDAO.class);
		if (null != categoriesAll && !categoriesAll.equalsIgnoreCase("true")) {
			String selectedCategories = (String) parameters.get("selectedCategory");
			String[] categories = selectedCategories.split(",");
			List categoryList = new ArrayList();
			if (null != categories && categories.length > 0) {
				for (int i = 0; i < categories.length; i++)
					categoryList.add(categories[i].trim());
				products = gatewayDAO.loadProductsForCategories(categoryList, firstID, lastID);
			}
		} else {
			products = gatewayDAO.loadAllProductsInRange(firstID, lastID);
		}
		if (null != products) {
			String productsAll = (String) parameters.get("productsAll");
			if (null != productsAll && !productsAll.equalsIgnoreCase("true")) {
				String productsStartCount = (String) parameters.get("productsStartCount");
				String productsEndCount = (String) parameters.get("productsEndCount");
				int startPoint = Integer.parseInt(productsStartCount);
				int endPoint = Integer.parseInt(productsEndCount);
				Object[] objProducts = products.toArray();
				if (startPoint < objProducts.length && endPoint > startPoint) {
					ArrayList list = new ArrayList();
					for (int i = startPoint; i < endPoint && i < objProducts.length; i++)
						list.add(objProducts[i]);
					products.retainAll(list);
				}
			}
		}
		return products;
	}

	public Collection processCategoryExport(Map parameters) throws Exception {
		int firstID = ((Integer) parameters.get("firstID")).intValue();
		int lastID = ((Integer) parameters.get("lastID")).intValue();
		Collection categories = null;
		CategoryGatewayDAO gatewayDAO =  getDaoFactory().createDAO(CategoryGatewayDAO.class);
		categories = gatewayDAO.loadAllCategoriesInRange(firstID, lastID);
		return categories;
	}

	public Collection processAttributeExport(Map parameters) throws Exception {
		int firstID = ((Integer) parameters.get("firstID")).intValue();
		int lastID = ((Integer) parameters.get("lastID")).intValue();
		Collection attributes = null;
		AttributeGatewayDAO gatewayDAO = (AttributeGatewayDAO) getDaoFactory().createDAO(
				"attributeGatewayDAOImplementer");
		attributes = gatewayDAO.loadAllAttributesInRange(firstID, lastID);
		return attributes;
	}

	public Collection processOptionExport(Map parameters) throws Exception {
		int firstID = ((Integer) parameters.get("firstID")).intValue();
		int lastID = ((Integer) parameters.get("lastID")).intValue();
		Collection options = null;
		OptionGatewayDAO gatewayDAO =  getDaoFactory().createDAO(OptionGatewayDAO.class);
		options = gatewayDAO.loadAllOptionsInRange(firstID, lastID);
		return options;
	}

	public Collection processSKUExport(Map parameters) throws Exception {
		int firstID = ((Integer) parameters.get("firstID")).intValue();
		int lastID = ((Integer) parameters.get("lastID")).intValue();
		Collection skus = null;
		SKUGatewayDAO gatewayDAO =  getDaoFactory().createDAO(SKUGatewayDAO.class);
		skus = gatewayDAO.loadAllSKUsInRange(firstID, lastID);
		return skus;
	}

	public Collection processOrderExport(Map parameters) throws Exception {
		String ordersAll = (String) parameters.get("ordersAll");
		String statusAll = (String) parameters.get("statusAll");
		int firstID = ((Integer) parameters.get("firstID")).intValue();
		int lastID = ((Integer) parameters.get("lastID")).intValue();
		String startDate = "1900-01-01"; // Setting to 01/01/1900
		String endDate = (new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault()).format(new java.util.Date()));
		if (null != ordersAll && !ordersAll.equalsIgnoreCase("true")) {
			startDate = (String) parameters.get("startDate");
			endDate = (String) parameters.get("endDate");
		}
		Collection orders = null;
		OrderGatewayDAO dao =  getDaoFactory().createDAO(OrderGatewayDAO.class);
		boolean allOrders = true;
		if (null != statusAll && !statusAll.equalsIgnoreCase("true"))
			allOrders = false;
		if (log.isDebugEnabled())
			log.debug("endDate: " + endDate);
		if (log.isDebugEnabled())
			log.debug("startDate: " + startDate);
		orders = dao.loadExport(startDate, endDate, allOrders, firstID, lastID);
		Collection results = new ArrayList();
		if (null != orders) {
			results = new ArrayList();
			Iterator i = orders.iterator();
			while (i.hasNext()) {
				Map row = (Map) i.next();
				OrdersExportBean oeb = (OrdersExportBean) getBusinessObjectFactory().createObject(
						"ordersExportImplementer");
				BeanUtils.populate(oeb, row);
				results.add(oeb);
			}
		}
		return results;
	}

	public Collection processCustomerExport(Map parameters) throws Exception {
		int firstID = ((Integer) parameters.get("firstID")).intValue();
		int lastID = ((Integer) parameters.get("lastID")).intValue();
		Collection customers = null;
		CustomerGatewayDAO gatewayDAO =  getDaoFactory().createDAO(CustomerGatewayDAO.class);
		customers = gatewayDAO.loadAllCustomersInRange(firstID, lastID);
		return customers;
	}

	public Collection processProductSettingExport(Map parameters) throws Exception {
		int firstID = ((Integer) parameters.get("firstID")).intValue();
		int lastID = ((Integer) parameters.get("lastID")).intValue();
		Collection productSettings = null;
		ProductGatewayDAO gatewayDAO =  getDaoFactory().createDAO(ProductGatewayDAO.class);
		productSettings = gatewayDAO.loadAllProductSettingsInRange(firstID, lastID);
		return productSettings;
	}

}
