package com.softslate.commerce.businessobjects.order;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with order items. Created
 * in the Struts layer in response to requests for viewing, editing, adding, and
 * deleting order items.
 * <p>
 * When the application needs to create an instance that implements
 * <code>OrderItemProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "orderItemProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "orderItemProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.OrderItemProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface OrderItemProcessor extends BusinessProcessor {

	abstract Map loadOrderItemsAndCount(Map parameters) throws Exception;

	abstract Map addOrderItem(Map parameters) throws Exception;

	abstract OrderItem loadOrderItemFromID(Map parameters) throws Exception;

	abstract Map editOrderItem(Map parameters) throws Exception;

	abstract Map editDeleteOrderItems(Map parameters) throws Exception;

	abstract void deleteOrderItem(Map parameters) throws Exception;
}
