package com.softslate.commerce.businessobjects.payment;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;
import com.softslate.commerce.businessobjects.order.Order;

/**
 * Interface representing a payment within the system. A payment contains some
 * of the important results of a successful result from a payment processor.
 * <p>
 * When the application needs to create an instance that implements
 * <code>sscPayment</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the "paymentImplementer"
 * setting in the <code>appComponents.properties</code> file.
 * <p>
 * The default "paymentImplementer" is
 * {@link com.softslate.commerce.businessobjects.payment.PaymentBean}.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudPayment")
public interface Payment extends BusinessObject {
	abstract int getPaymentID();

	abstract void setPaymentID(int paymentID);

	abstract String getCreditCardDate();

	abstract void setCreditCardDate(String creditCardDate);

	abstract String getCreditCardName();

	abstract void setCreditCardName(String creditCardName);

	abstract String getDecryptedNumber();

	abstract void setDecryptedNumber(String decryptedNumber);

	abstract String getCreditCardNumber();

	abstract void setCreditCardNumber(String creditCardNumber);

	abstract String getCreditCardNumberDisplay();

	abstract void setCreditCardNumberDisplay(String creditCardNumberDisplay);

	abstract String getCreditCardType();

	abstract void setCreditCardType(String creditCardType);

	abstract int getOrderID();

	abstract void setOrderID(int orderID);

	abstract String getProcessorClassName();

	abstract void setProcessorClassName(String processorClassName);

	abstract String getStatus();

	abstract void setStatus(String status);

	abstract String getValue1();

	abstract void setValue1(String value1);

	abstract String getValue2();

	abstract void setValue2(String value2);

	abstract String getValue3();

	abstract void setValue3(String value3);

	abstract String getValue4();

	abstract void setValue4(String value4);

	abstract String getValue5();

	abstract void setValue5(String value5);

	abstract String getValue6();

	abstract void setValue6(String value6);

	abstract String getValue7();

	abstract void setValue7(String value7);

	abstract String getValue8();

	abstract void setValue8(String value8);

	abstract String getValue9();

	abstract void setValue9(String value9);

	abstract String getCreated();

	abstract void setCreated(String created);

	abstract Double getAmount();

	abstract void setAmount(Double amount);

	abstract String getValueFieldLabels();

	abstract void setValueFieldLabels(String valueFieldLabels);

	abstract String getExtra1();

	abstract void setExtra1(String extra1);

	abstract String getExtra2();

	abstract void setExtra2(String extra2);

	abstract String getExtra3();

	abstract void setExtra3(String extra3);

	abstract String getPaymentProfileID();

	abstract void setPaymentProfileID(String paymentProfileID);

	abstract Order getOrder();

	abstract void setOrder(Order order);
}
