package com.softslate.commerce.businessobjects.product;

import java.io.Serializable;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing a discount range within the system.
 * <code>DiscountRangeBean</code> is the default implementation of the
 * <code>DiscountRange</code> interface for the application.
 * <p>
 * The database stores discount ranges in the <code>sscDiscountRange</code>
 * table, and this interface can be used to represent a single row of that
 * table. The Hibernate configuration file
 * <code>DiscountRangeBean.hbm.xml</code> maps that table's columns to the
 * properties in this object.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudDiscountRange")
public class DiscountRangeBean extends BaseBusinessObject implements
		DiscountRange, Serializable {

	private static final long serialVersionUID = 6892250037420953666L;

	private int discountRangeID = 0;

	private int discountID = 0;

	private Discount discount = null;

	private String rewardType = null;

	private Double reward = null;

	private Double floor = null;

	private Double ceiling = null;

	private boolean isActive = true;

	public Double getCeiling() {
		return ceiling;
	}

	public void setCeiling(Double ceiling) {
		this.ceiling = ceiling;
	}

	public Discount getDiscount() {
		return discount;
	}

	public void setDiscount(Discount discount) {
		this.discount = discount;
	}

	public int getDiscountID() {
		return discountID;
	}

	public void setDiscountID(int discountID) {
		this.discountID = discountID;
	}

	public int getDiscountRangeID() {
		return discountRangeID;
	}

	public void setDiscountRangeID(int discountRangeID) {
		this.discountRangeID = discountRangeID;
	}

	public Double getFloor() {
		return floor;
	}

	public void setFloor(Double floor) {
		this.floor = floor;
	}

	public boolean getIsActive() {
		return isActive;
	}

	public void setIsActive(boolean isActive) {
		this.isActive = isActive;
	}

	public Double getReward() {
		return reward;
	}

	public void setReward(Double reward) {
		this.reward = reward;
	}

	public String getRewardType() {
		return rewardType;
	}

	public void setRewardType(String rewardType) {
		this.rewardType = rewardType;
	}

}
