package com.softslate.commerce.businessobjects.product;

import java.util.Collection;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for classes that search on the Lucene Suggestions Index for suggestions to help users search.
 * 
 * When the application needs to create an instance that implements <code>LuceneSuggestionsSearcher</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory} finds the name of the Java class to
 * instantiate from the "luceneSuggestionsSearcherImplementer" setting in the <code>appComponents.properties</code>
 * file.
 * <p>
 * The default "luceneSuggestionsSearcherImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.BasicLuceneSuggestionsSearcher}.
 * 
 * @author David Tobey
 * 
 */
public interface LuceneSuggestionsSearcher extends BusinessProcessor {

	public Collection searchSuggestions(String indexDir, String searchString) throws Exception;
}
