package com.softslate.commerce.businessobjects.product;

import java.util.Collection;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing a manufacturer within the system. A manufacturer is a
 * company that makes or provides products as a vendor.
 * <p>
 * When the application needs to create an instance that implements
 * <code>Manufacturer</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "manufacturerImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "manufacturerImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.ManufacturerBean}.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudManufacturer")
public interface Manufacturer extends BusinessObject {

	@NotAudited
	abstract Collection getProducts();

	abstract void setProducts(Collection products);

	abstract String getCode();

	abstract void setCode(String code);

	abstract String getDescription();

	abstract void setDescription(String description);

	abstract String getExtra1();

	abstract void setExtra1(String extra1);

	abstract String getExtra2();

	abstract void setExtra2(String extra2);

	abstract String getExtra3();

	abstract void setExtra3(String extra3);

	abstract String getFooter();

	abstract void setFooter(String footer);

	abstract String getHeader();

	abstract void setHeader(String header);

	abstract boolean getIsActive();

	abstract void setIsActive(boolean isActive);

	abstract String getLargeImage();

	abstract void setLargeImage(String largeImage);

	abstract int getManufacturerID();

	abstract void setManufacturerID(int manufacturerID);

	abstract int getManufacturerOrder();

	abstract void setManufacturerOrder(int manufacturerOrder);

	abstract String getName();

	abstract void setName(String name);

	abstract String getSmallImage();

	abstract void setSmallImage(String smallImage);

	abstract String getWebsiteURL();

	abstract void setWebsiteURL(String websiteURL);

	public String getAddress1();

	public void setAddress1(String address1);

	public String getAddress2();

	public void setAddress2(String address2);

	public String getCity();

	public void setCity(String city);

	public String getState();

	public void setState(String state);

	public String getPostalCode();

	public void setPostalCode(String postalCode);
	
	public String getCountry();

	public void setCountry(String country);

	public String getEmail();

	public void setEmail(String email);

	public String getPhone();

	public void setPhone(String phone);

	public boolean getIsDropShipper();

	public void setIsDropShipper(boolean isDropShipper);

	abstract int getItemsPerPage();

	abstract void setItemsPerPage(int itemsPerPage);

	abstract int getFirstRow();

	abstract void setFirstRow(int firstRow);

	abstract int getItemCount();

	abstract void setItemCount(int itemCount);

	abstract Collection getProductListCollection();

	abstract void setProductListCollection(Collection products);

}
