package com.softslate.commerce.customer.customer;

import java.util.Collection;
import java.util.Map;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.businessobjects.customer.CustomerWishList;
import com.softslate.commerce.businessobjects.customer.CustomerWishListProduct;
import com.softslate.commerce.businessobjects.customer.CustomerWishListProcessor;
import com.softslate.commerce.businessobjects.product.ProductList;
import com.softslate.commerce.businessobjects.product.ProductListBean;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to the wish list page. Used for requests
 * to "/WishList.do".
 * <p>
 * The <code>execute</code> method of this class populates wish list information
 * from the database and puts it in the request and session scopes for use by
 * the view layer. Given a
 * {@link com.softslate.commerce.customer.customer.WishListAddEditForm} with its
 * <code>id</code> property populated, it creates and initializes an instance of
 * {@link com.softslate.commerce.businessobjects.customer.CustomerWishList}.
 * 
 * @author Jason McWilliams
 * 
 * @see com.softslate.commerce.customer.customer.WishListAddEditForm
 * @see com.softslate.commerce.businessobjects.customer.CustomerWishList
 * @see com.softslate.commerce.businessobjects.customer.CustomerWishListProcessor
 */
public class WishListAction extends BaseAction {

	static Log log = LogFactory.getLog(WishListAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");

		WishListForm baseForm = (WishListForm) form;

		// Check Settings
		// If the system is not allowing wish lists, error
		Settings settings = (Settings) servlet.getServletContext()
				.getAttribute("settings");
		String allowCustomerWishLists = (String) settings
				.getValue("allowCustomerWishLists");
		if (allowCustomerWishLists == null
				|| !allowCustomerWishLists.equals("1")) {
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.wishListsNotAllowed"));
			saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
					.getErrors());
			if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
				log.info("Validation error(s): "
						+ baseForm.getErrors().toString());
			return mapping.findForward("failure");
		}

		// Initialize a CustomerWishList object
		CustomerWishListProcessor customerWishListProcessor = (CustomerWishListProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"customerWishListProcessorImplementer");

		// Load the wish list from the ID
		CustomerWishList customerWishList = null;
		int wishID = 0;
		try {
			wishID = Integer.parseInt(baseForm.getWishID());
		} catch (NumberFormatException nfe) {
			wishID = 0;
		}

		if (wishID > 0) {
			Map parameters = new HashMap();
			parameters.put("customerWishListID", new Integer(wishID));
			customerWishList = customerWishListProcessor
					.loadCustomerWishListFromID(parameters);
		}

		// If the wish list was not found or it does not pass the check, error.
		if (customerWishList == null
				|| customerWishList.getCustomerWishListID() == 0
				|| baseForm.getCheck() == null
				|| !baseForm.getCheck().equals(
						customerWishListProcessor
								.computeCheck(customerWishList))) {
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.wishListNotFound"));
			saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
					.getErrors());
			if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
				log.info("Validation error(s): "
						+ baseForm.getErrors().toString());
			return mapping.findForward("failure");
		}

		// Set the product list parameters
		String itemsPerPage = (String) baseForm.getSettings().getValue(
				"wishListPageSize");
		if (log.isDebugEnabled())
			log.debug("itemsPerPage: " + itemsPerPage);
		customerWishList.setItemsPerPage(Integer.parseInt(itemsPerPage));
		int firstRow = 1;
		try {
			firstRow = Integer.parseInt(baseForm.getFirstRow());
		} catch (NumberFormatException nfe) {
			firstRow = 1;
		}
		customerWishList.setFirstRow(firstRow);

		// Build the product collection manually
		Collection products = new ArrayList();
		Iterator i = customerWishList.getCustomerWishListProducts().iterator();
		int count = 1;
		int counted = 0;
		while (i.hasNext()) {
			CustomerWishListProduct customerProductListProduct = (CustomerWishListProduct) i
					.next();
			if ((count >= firstRow)
					&& (counted <= customerWishList.getItemsPerPage())) {
				products.add(customerProductListProduct.getProduct());
				counted++;
			}
			count++;
		}
		customerWishList.setItemCount(customerWishList
				.getCustomerWishListProducts().size());
		ProductList productList = (ProductList) baseForm
				.getBusinessObjectFactory().createObject(
						"productListImplementer");
		productList.setProductListType(ProductListBean.WISH_LIST);
		productList.setCustomerWishListID(customerWishList
				.getCustomerWishListID());
		productList.setItemsPerPage(customerWishList.getItemsPerPage());
		productList.setFirstRow(customerWishList.getFirstRow());
		productList.setItemCount(customerWishList.getItemCount());
		productList.setProductListCollection(products);
		productList.addFormatting(baseForm.getUser().getLocale(),
				(String) baseForm.getSettings().getValue("currencyCode"));
		customerWishList.setProductListCollection(productList
				.getProductListCollection());

		// Load all attribute-only SKUs
		ProductProcessor productProcessor = (ProductProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"productProcessorImplementer");
		String trackInventory = (String) baseForm.getSettings().getValue(
				"trackInventory");
		String applyDiscounts = (String) baseForm.getSettings().getValue(
				"applyDiscounts");
		if ((trackInventory != null && trackInventory.equals("1"))
				|| (applyDiscounts != null && applyDiscounts.equals("1"))) {
			Map attributeSKUs = productProcessor.loadAttributeSKUs();
			if (log.isDebugEnabled())
				log.debug("Atribute-only skus: " + attributeSKUs);
			request.setAttribute("attributeSKUs", attributeSKUs);
			Collection attributeSKUCollection = productProcessor
					.loadAttributeSKUCollection();
			request.setAttribute("attributeSKUCollection",
					attributeSKUCollection);
		}

		// Save variable in the request scope for JSPs to use
		request.setAttribute("customerWishList", customerWishList);
		request.setAttribute("customerWishLists", customerWishList
				.getCustomer().getCustomerWishLists());

		// Set various meta tag fields
		if (customerWishList.getName() != null) {
			String subTitle = customerWishList.getName().replaceAll("\\<.*?>",
					"");
			if (subTitle.length() > 0) {
				request.setAttribute("htmlHeadTitle", request
						.getAttribute("htmlHeadTitle")
						+ " - " + subTitle);
			}
		}
		if (customerWishList.getDescription() != null) {
			String desc = customerWishList.getDescription().replaceAll(
					"\\<.*?>", "");
			if (desc.length() > 0) {
				request.setAttribute("htmlHeadDescription", desc);
			}
		}

		return mapping.findForward("success");
	}

}
