package com.softslate.commerce.customer.order;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.order.OrderItem;

/**
 * Struts form class for submissions to edit an item in the current user's cart.
 * Used for requests to "/CartItemEdit.do".
 * <p>
 * The <code>validate</code> method of this class extends
 * {@link com.softslate.commerce.customer.order.CartAddForm#validate(ActionMapping, HttpServletRequest)}.
 * It checks the <code>orderItemID</code> property to make sure it exists;
 * checks the <code>quantity</code> property to make sure it's greater than
 * zero; and checks to make sure the user's session has not expired. In doing
 * this, it also populates the <code>orderItem</code> property with the
 * {@link com.softslate.commerce.businessobjects.order.OrderItem} instance being
 * edited.
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.customer.order.CartItemEditAction
 */
public class CartItemEditForm extends CartAddForm {

	private static final long serialVersionUID = 3258133552762925108L;

	static Log log = LogFactory.getLog(CartItemEditForm.class);

	private String orderItemID = null;

	private OrderItem orderItem;

	public OrderItem getOrderItem() {
		return orderItem;
	}

	public void setOrderItem(OrderItem orderItem) {
		this.orderItem = orderItem;
	}

	public String getOrderItemID() {
		return orderItemID;
	}

	public void setOrderItemID(String orderItemID) {
		this.orderItemID = orderItemID;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);

		// Check for catalog mode
		try {
			String catalogMode = (String) getSettings().getValue("catalogMode");
			if (catalogMode.equals("1")) {
				getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.catalogMode"));
				return getErrors();
			}
		} catch (Exception e) {
		}

		// Check that at least one product code was submitted.
		validateProductCode();

		// Validate attributes
		validateAttributes();

		// Check that an order item id is present.
		if (!formUtils().isInteger(getOrderItemID())) {
			String message = getMessage("order", "cartItemEdit.item", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}

		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}

}
