package com.softslate.commerce.customer.order;

import java.util.Collection;
import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.order.OrderItem;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts form class for requests for the cart item edit form for a given cart
 * item. Used for requests to "/CartItemEditForm.do".
 * <p>
 * The <code>validate</code> method of this class checks the
 * <code>orderItemID</code> property to make sure it exists. It also checks to
 * make sure user's session has not expired, and that the given order item is in
 * the user's cart. In doing this, it also populates the <code>orderItem</code>
 * property with the
 * {@link com.softslate.commerce.businessobjects.order.OrderItem} instance being
 * edited.
 * 
 * @author David Tobey
 * 
 */
public class CartItemEditFormForm extends BaseForm {

	private static final long serialVersionUID = 3258135760342496056L;

	static Log log = LogFactory.getLog(CartItemEditFormForm.class);

	private String orderItemID;

	private OrderItem orderItem;

	public OrderItem getOrderItem() {
		return orderItem;
	}

	public void setOrderItem(OrderItem orderItem) {
		this.orderItem = orderItem;
	}

	public String getOrderItemID() {
		return orderItemID;
	}

	public void setOrderItemID(String orderItemID) {
		this.orderItemID = orderItemID;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);

		// Check that an order item id is present.
		if (!formUtils().isInteger(getOrderItemID())) {
			String message = getMessage("order", "cartItemEdit.item", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}

		// Check that the user's session has not expired.
		if (getUser() == null || getUser().getOrder() == null) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.sessionExpired"));
		} else if (formUtils().isInteger(getOrderItemID())) {

			// Check that the order item is in the user's cart.
			int orderItemID = Integer.parseInt(getOrderItemID());

			// Try to find the OrderItem
			if (getUser().getOrder().firstDelivery() != null
					&& getUser().getOrder().firstDelivery()
							.getOrderItemCollection() != null) {
				Collection ois = getUser().getOrder().firstDelivery()
						.getOrderItemCollection();
				Iterator i = ois.iterator();
				while (i.hasNext()) {
					OrderItem oi = (OrderItem) i.next();
					if (oi.getOrderItemID() == orderItemID) {
						setOrderItem(oi);
						break;
					}
				}
			}
			// Error if the OrderItem can't be found
			if (getOrderItem() == null) {
				getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.orderItemNotFound"));
			}
		}
		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}

}
