package com.softslate.commerce.customer.payment;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

/**
 * Struts form class for submissions of bank account information for PayPal Payflow
 * Pro's ACH processing during checkout. This form corresponds to the
 * {@link com.softslate.commerce.businessobjects.payment.PayflowProACHProcessor}
 * payment processor.
 * <p>
 * The database settings "activePaymentProcessors", "activePaymentForms", and
 * "activePaymentFormTemplate" represent the various payment methods available
 * to users. During checkout, each of the "activePaymentForms" will be
 * instantiated and prepared for display on the checkout payment screen. When
 * the user submits payment information the <code>validate</code> method of
 * the corresponding payment form is invoked.
 * <p>
 * The <code>validate</code> method of this class ensures that the incoming
 * bank account information is valid and complete. It then parses the database
 * settings for various Payflow Pro fields and populates the corresponding form
 * properties. It also sets the cert path for Payflow Pro equal to the location
 * of /WEB-INF/conf/keys on the server.
 * 
 * @author Ashish Jain
 */
public class PayflowProACHForm extends PayflowProForm {

	static Log log = LogFactory.getLog(PayflowProACHForm.class);

	private static final long serialVersionUID = 610713101752844498L;

	public static final String ACCT_TYPE_CHECKING = "C";

	public static final String ACCT_TYPE_SAVINGS = "S";

	public static final String validAcctTypes[] = { ACCT_TYPE_CHECKING,
			ACCT_TYPE_SAVINGS };

	public static final boolean isABARoutingNumberRequired = true;

	public static final boolean isAccountNumberRequired = true;

	public static final boolean isAccountTypeRequired = true;

	private String nameOnAccount = null;

	private String ABARoutingNumber = null;

	private String acctNumber = null;

	private String acctType = null;

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isDebugEnabled())
			log.info("Starting validation.");

		initializeProperties(mapping, request);
		payflowProACHValidate();
		return payflowValidate();
	}

	protected void payflowProACHValidate() {
		this.validateNameOnAccount();
		this.validateABANumber();
		this.validateAcctType();
		this.validateAcctNumber();
	}

	protected void validateNameOnAccount() {
		String message = getMessage("order", "paymentForms.ACHName", null);

		if (this.getNameOnAccount() == null
				| this.getNameOnAccount().length() == 0)
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
	}

	protected void validateABANumber() {
		String message = getMessage("order", "paymentForms.ACHABANumber", null);
		this.setABARoutingNumber(this.validateNumberCommon(this
				.getABARoutingNumber(), isABARoutingNumberRequired, 0, 9,
				message));
	}

	protected void validateAcctNumber() {
		String message = getMessage("order", "paymentForms.ACHAcctNumber", null);
		this.setAcctNumber(this.validateNumberCommon(this.getAcctNumber(),
				isAccountNumberRequired, 0, 17, message));
	}

	protected void validateAcctType() {
		for (int i = 0; i < validAcctTypes.length; i++)
			if (this.getAcctType().equals(validAcctTypes[i]))
				return;

		String message = getMessage("order", "paymentForms.ACHAcctType", null);
		getErrors().add(ActionMessages.GLOBAL_MESSAGE,
				new ActionMessage("errors.invalid", message));
	}

	public String validateNumberCommon(String number, boolean isReq,
			int minLength, int maxLength, String message) {

		if (number != null)
			number = number.replaceAll("\\D", "");

		if (isReq) {
			// String message = getMessage("order",
			// "paymentForms.ccSecurityCode", null);
			if (number == null || number.length() == 0) {
				getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.required", message));
			} else if (number.length() > maxLength
					|| number.length() < minLength) {
				getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.invalid", message));
			}
		}
		return number;
	}

	public String getABARoutingNumber() {
		return ABARoutingNumber;
	}

	public void setABARoutingNumber(String routingNumber) {
		ABARoutingNumber = routingNumber;
	}

	public String getAcctNumber() {
		return acctNumber;
	}

	public void setAcctNumber(String acctNumber) {
		this.acctNumber = acctNumber;
	}

	public String getNameOnAccount() {
		return nameOnAccount;
	}

	public void setNameOnAccount(String nameOnAccount) {
		this.nameOnAccount = nameOnAccount;
	}

	public String getAcctType() {
		return acctType;
	}

	public void setAcctType(String acctType) {
		this.acctType = acctType;
	}

}
