package com.softslate.commerce.customer.product;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.product.Category;
import com.softslate.commerce.businessobjects.product.LuceneSearcher;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.businessobjects.product.SearchList;
import com.softslate.commerce.businessobjects.product.SearchListBean;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to the search results page. Used for
 * requests to "/AdvancedSearch.do".
 * <p>
 * The <code>execute</code> method of this class retrieves search results from
 * the lucene index and then grabs the appropriate collection of products from
 * the database to make sure all the product information is available for
 * display. One could display the results directly from Lucene as it does have
 * id, code, name, description, etc. It puts it in the request scope for use by
 * the view layer. Given a
 * {@link com.softslate.commerce.customer.product.AdvancedSearchForm} with its
 * <code>searchString</code> property populated, it creates and initializes an
 * instance of
 * {@link com.softslate.commerce.businessobjects.product.ProductList}. It then
 * invokes
 * {@link com.softslate.commerce.businessobjects.product.LuceneSearcher#getSearchList}
 * to load the product list information from the database. It puts the
 * initialized <code>ProductList</code> object in the request scope as an
 * attribute with the name "productList".
 * 
 * @author Jason McWilliams, David Tobey
 * 
 * @see com.softslate.commerce.customer.product.AdvancedSearchForm
 * @see com.softslate.commerce.businessobjects.product.ProductList
 * @see com.softslate.commerce.businessobjects.product.LuceneSearcher
 */
public class AdvancedSearchAction extends BaseAction {

	static Log log = LogFactory.getLog(SearchAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		AdvancedSearchForm baseForm = (AdvancedSearchForm) form;

		// Don't do it if its not turned on
		String advancedSearchOn = (String) baseForm.getSettings().getValue("advancedSearchOn");
		if (!advancedSearchOn.equals("1")) {
			return mapping.findForward("failure");
		}

		// First create the processor to do the searching
		LuceneSearcher luceneSearcher = (LuceneSearcher) baseForm.getBusinessObjectFactory().createObject(
				"luceneSearcherImplementer");

		// Create a SearchList helper bean
		SearchList searchList =  baseForm.getBusinessObjectFactory().createObject(SearchList.class);
		String itemsPerPage = (String) baseForm.getSettings().getValue("searchResultsPageSize");
		if (baseForm.getFirstRow() != null && baseForm.getFirstRow().equals("all")) {
			baseForm.setFirstRow("1");
			itemsPerPage = (String) baseForm.getSettings().getValue("maxRecordsProcessed");
		}
		if (log.isDebugEnabled())
			log.debug("itemsPerPage: " + itemsPerPage);
		searchList.setItemsPerPage(Integer.parseInt(itemsPerPage));
		BeanUtils.copyProperties(searchList, baseForm);
		baseForm.formUtils().eliminateDuplicateFacets(searchList);

		// Change the type and set the category id
		int searchListType = SearchListBean.SEARCH_PRODUCTS;
		int categoryID = 0;
		try {
			int type = Integer.parseInt(baseForm.getType());
			searchListType = type;
			int cat = Integer.parseInt(baseForm.getCategoryID());
			categoryID = cat;
		} catch (Exception ex) {
			searchListType = SearchListBean.SEARCH_PRODUCTS;
			categoryID = 0;
		}
		searchList.setCategoryID(categoryID);
		searchList.setSearchListType(searchListType);

		// Look up the category id if a categoryCode was submitted
		if (categoryID == 0 && baseForm.getCategoryCode() != null && baseForm.getCategoryCode().length() > 0) {
			ProductProcessor productProcessor = (ProductProcessor) baseForm.getBusinessObjectFactory().createObject(
					"productProcessorImplementer");
			Category category =  baseForm.getBusinessObjectFactory().createObject(Category.class);
			category.setCode(baseForm.getCategoryCode());
			category = productProcessor.getCategoryFromCode(category, false);
			searchList.setCategoryID(category.getCategoryID());
		}
		searchList.setCategoryMap((Map) getServlet().getServletContext().getAttribute("categoryMap"));

		// Load the objects and place the list in the request scope
		searchList = luceneSearcher.getSearchList(searchList);
		request.setAttribute("searchList", searchList);
		request.setAttribute("advancedSearch", "true");
		request.setAttribute("itemCount", new Integer(searchList.getItemCount()));
		request.setAttribute("itemsPerPage", new Integer(searchList.getItemsPerPage()));
		request.setAttribute("firstRow", new Integer(searchList.getFirstRow()));
		request.setAttribute("listCollection", searchList.getSearchListCollection());
		request.setAttribute("objectTypes", searchList.getObjectTypes());
		request.setAttribute("objectScores", searchList.getObjectScores());
		request.setAttribute("objectExcerpts", searchList.getObjectExcerpts());
		request.setAttribute("searchType", searchList.getSearchListType());
		request.setAttribute("categoryID", searchList.getCategoryID());
		String sort = searchList.getSortBy();
		request.setAttribute("sortByParam", "&sortBy=" + java.net.URLEncoder.encode(sort, "UTF-8"));
		String searchString = request.getParameter("searchString");
		String facetsParam = baseForm.formUtils().stringArrayAsURLParam("facets", searchList.getFacets());
		request.setAttribute("facetsParam", facetsParam);
		String linkBase = "/AdvancedSearch.do?searchString="
				+ java.net.URLEncoder.encode(searchString, "UTF-8") + "&amp;type=" + searchList.getSearchListType()
				+ "&amp;categoryID=" + searchList.getCategoryID() + "&amp;";
		request.setAttribute("linkBase", linkBase);

		// Forward to Ajax tile if it's an Ajax request (faceted search)
		if (baseForm.getAjax() != null && baseForm.getAjax().equals("true")) {
			if (("default-html5").equals(baseForm.html5OrXhtml())) {
				return mapping.findForward("successAjaxHTML5");
			} else {
				return mapping.findForward("successAjax");
			}
		}

		return mapping.findForward("success");
	}

}
