package com.softslate.commerce.customer.product;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.customer.core.BaseForm;
import javax.servlet.http.HttpServletRequest;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.upload.FormFile;

/**
 * This class is used for the presentation/struts side of the product review
 * form submission.
 * 
 * @author Tony Zakula
 */
public class ProductReviewForm extends BaseForm {

	private static final long serialVersionUID = 1L;

	static Log log = LogFactory.getLog(ProductReviewForm.class);

	private String firstName = null;
	private String lastName = null;
	private String age = null;
	private String email = null;
	private String city = null;
	private String state = null;
	private String review = null;
	private Integer rating = null;
	private String imagePath = null;
	private String country = null;
	private FormFile imageFile = null;
	private String code = null;
	private String nickname = null;

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);

		if (getNickname() == null || getNickname().length() == 0) {
			String message = "Nickname ";
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}
		
		// Strip html characters from the review
		setReview(getReview().replaceAll("\\<.*?>", ""));

		if (getRating() == null || getRating().intValue() == 0) {
			String message = "Rating";
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}

		if (getCode() == null || getCode().length() == 0) {
			String message = "Product code";
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}

		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getCity() {
		return city;
	}

	public void setCity(String city) {
		this.city = city;
	}

	public String getState() {
		return state;
	}

	public void setState(String state) {
		this.state = state;
	}

	public String getReview() {
		return review;
	}

	public void setReview(String review) {
		this.review = review;
	}

	public Integer getRating() {
		return rating;
	}

	public void setRating(Integer rating) {
		this.rating = rating;
	}

	public String getImagePath() {
		return imagePath;
	}

	public void setImagePath(String imagePath) {
		this.imagePath = imagePath;
	}

	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public String getAge() {
		return age;
	}

	public void setAge(String age) {
		this.age = age;
	}

	public String getCountry() {
		return country;
	}

	public void setCountry(String country) {
		this.country = country;
	}

	public FormFile getImageFile() {
		return imageFile;
	}

	public void setImageFile(FormFile imageFile) {
		this.imageFile = imageFile;
	}

	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	public String getNickname() {
		return nickname;
	}

	public void setNickname(String nickname) {
		this.nickname = nickname;
	}

}