package com.softslate.commerce.daos.core;

import org.hibernate.envers.RevisionListener;

import com.softslate.commerce.businessobjects.core.RequestLog;
import com.softslate.commerce.businessobjects.core.RevisionEntityBean;

/**
 * Record the sscRequestLog.requestLogID with each Envers revision. From the RequestLog record, you can retrieve the
 * userName, IP address, URL, etc. Request logging must be enabled with the "Log Requests to Database" setting on the
 * Advanced System Settings screen (Enterprise Edition only), otherwise auditing can still be used but null will be
 * written for all requestIDs.
 * 
 * To use a different RevisionListener than this, override the "revisionEntityImplementer" property in
 * appComponents-custom.properties with a custom implementation of RevisionEntity, replacing RevisionEntityBean. In the
 * new implementation, use the @RevisionEntity annotation to specify a new listener.
 * 
 * @author David Tobey
 *
 */
public class AppRevisionListener implements RevisionListener {

	private static final ThreadLocal threadRequestLog = new ThreadLocal();

	public static void setRequestLogForThread(RequestLog rl) {
		threadRequestLog.set(rl);
	}

	public void newRevision(Object revisionEntity) {
		RevisionEntityBean revEntity = (RevisionEntityBean) revisionEntity;
		RequestLog rl = (RequestLog) threadRequestLog.get();
		if (rl != null) {
			revEntity.setRequestLogID(rl.getRequestLogID());
		}
	}
}
