package com.softslate.commerce.daos.email;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.email.Template;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;

public class TemplateDAOHibernate extends BaseDAO implements DAOInterface, TemplateDAO {

    static Log log = LogFactory.getLog(TemplateDAOHibernate.class);

    private Template template = null;

    public Template getTemplate() {
        return (this.template);
    }

    public void setTemplate(Template template) {
        this.template = template;
    }

    public void loadTemplateFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            Template record = (Template) session.get(
                    Template.class, new Integer(getTemplate()
                            .getTemplateID()));
            PropertyUtils.copyProperties(getTemplate(), record);
            setTemplate(record);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void insertTemplate() throws Exception {
        insertTemplate(false);
    }

    public void insertTemplate(boolean commit) throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            session.save(getTemplate());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
			if (getTemplate() != null) {
				session.evict(getTemplate());
			}
			loadTemplateFromID();
        } catch (HibernateException he) {
            if (log.isErrorEnabled()) log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateTemplate() throws DataAccessException {
        updateTemplate(false);
    }

    public void updateTemplate(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.update(getTemplate());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled()) log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteTemplate() throws Exception {
        deleteTemplate(false);
    }

    public void deleteTemplate(boolean commit) throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            setTemplate((Template) session.load(Template.class,
                    new Integer(getTemplate().getTemplateID())));
            session.delete(getTemplate());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) { log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }
}
