package com.softslate.commerce.daos.order;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.order.Country;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscCountry</code> table of the database.
 * 
 * @author David Tobey
 * 
 */
public class CountryDAOHibernate extends BaseDAO implements DAOInterface,
        CountryDAO {

    static Log log = LogFactory.getLog(CountryDAOHibernate.class);

    private Country country = null;

    public Country getCountry() {
        return (this.country);
    }

    public void setCountry(Country country) {
        this.country = country;
    }

    public void loadCountryFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            Country record = (Country) session.get(Country.class, new Integer(
                    getCountry().getCountryID()));
            PropertyUtils.copyProperties(getCountry(), record);
            setCountry(record);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void insertCountry() throws DataAccessException {
        insertCountry(false);
    }

    public void insertCountry(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.save(getCountry());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateCountry() throws DataAccessException {
        updateCountry(false);
    }

    public void updateCountry(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.update(getCountry());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteCountry() throws DataAccessException {
        deleteCountry(false);
    }

    public void deleteCountry(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            setCountry((Country) session.load(Country.class, new Integer(
                    getCountry().getCountryID())));
            session.delete(getCountry());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }
}
