package com.softslate.commerce.daos.tax;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.tax.BasicTaxRate;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;


/**
 * Generic DAO to manage the <code>sscBasicTaxRate</code> table of the
 * database.
 * 
 * @author David Tobey
 * 
 */
public class BasicTaxRateDAOHibernate extends BaseDAO implements DAOInterface,
        BasicTaxRateDAO {

    static Log log = LogFactory.getLog(BasicTaxRateDAOHibernate.class);

    private BasicTaxRate basicTaxRate = null;

    public BasicTaxRate getBasicTaxRate() {
        return (this.basicTaxRate);
    }

    public void setBasicTaxRate(BasicTaxRate basicTaxRate) {
        this.basicTaxRate = basicTaxRate;
    }

    public void loadBasicTaxRateFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            BasicTaxRate record = (BasicTaxRate) session.get(
                    BasicTaxRate.class, new Integer(getBasicTaxRate()
                            .getBasicTaxRateID()));
            PropertyUtils.copyProperties(getBasicTaxRate(), record);
            setBasicTaxRate(record);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void insertBasicTaxRate() throws DataAccessException {
        insertBasicTaxRate(false);
    }

    public void insertBasicTaxRate(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.save(getBasicTaxRate());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateBasicTaxRate() throws DataAccessException {
        updateBasicTaxRate(false);
    }

    public void updateBasicTaxRate(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.update(getBasicTaxRate());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteBasicTaxRate() throws Exception {
        deleteBasicTaxRate(false);
    }

    public void deleteBasicTaxRate(boolean commit) throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            setBasicTaxRate((BasicTaxRate) session.load(BasicTaxRate.class,
                    new Integer(getBasicTaxRate().getBasicTaxRateID())));
            session.delete(getBasicTaxRate());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }
}
