package com.softslate.commerce.installer;

import java.io.File;
import java.util.Properties;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.DynaActionForm;

import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.businessobjects.core.SettingsBean;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.daos.core.DAOFactory;

/**
 * Do all the necessary functions required to process an installation.
 * 
 * @author David Tobey
 */
public class BasicFormAction extends BaseAction {

	static Log log = LogFactory.getLog(BasicFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		DynaActionForm dynaForm = (DynaActionForm) form;

		// Where are we at with the installation process? Define variables that
		// represent the step that have been accomplished.
		boolean licenseKeyInstalled = false;
		boolean twoWayKeyFileExists = false;
		boolean connectedToDatabase = false;
		boolean databaseObjectsCreated = false;
		boolean logFileDirectoryConfigured = false;
		boolean installerLocked = false; // Never locked if we got here

		// Is a License Key installed?
		String appSettingsPropertiesFile = "appSettings.properties";
		if (System.getProperty("appSettings.configuration") != null) {
			appSettingsPropertiesFile = System
					.getProperty("appSettings.configuration");
		}
		Properties appSettings = AppUtils.loadProperties(
				appSettingsPropertiesFile, getServlet().getServletContext());
		String licenseKey = appSettings.getProperty("licenseKey");
		if (licenseKey != null && licenseKey.length() > 0) {
			licenseKeyInstalled = true;
		}

		// Does the two way key file exist?
		String twoWayKeyFile = appSettings.getProperty("twoWayKeyFile");
		if (twoWayKeyFile != null && twoWayKeyFile.length() > 0) {
			File keyFile = new File(twoWayKeyFile);
			if (keyFile != null && keyFile.exists()) {
				twoWayKeyFileExists = true;
			}
		}

		// Are we connected to the database?
		String hibernatePropertiesFile = "hibernate.properties";
		if (System.getProperty("hibernate.configuration") != null) {
			hibernatePropertiesFile = System
					.getProperty("hibernate.configuration");
		}
		Properties hibernateSettings = AppUtils.loadProperties(
				hibernatePropertiesFile, getServlet().getServletContext());
		String dbURL = hibernateSettings
				.getProperty("hibernate.connection.url");
		DAOFactory daoFactory = (DAOFactory) servlet.getServletContext()
				.getAttribute("daoFactory");
		if (dbURL != null && !dbURL.endsWith("databasename")
				&& daoFactory != null) {
			try {
				daoFactory.startSession();
				connectedToDatabase = daoFactory.getSession().isConnected();
			} catch (Exception sqle) {
				log.warn(
						"Installer found connection to db failed: "
								+ sqle.toString(), sqle);
			}
		}

		// Have the database objects been created?
		Settings settings = (Settings) servlet.getServletContext()
				.getAttribute("settings");
		if (connectedToDatabase) {
			if (settings == null || settings.getMap() == null
					|| settings.getMap().isEmpty()) {
				try {
					SettingsBean settingsBean = new SettingsBean();
					settingsBean.setDaoFactory(daoFactory);
					settingsBean.initialize();
					settings = settingsBean;
				} catch (Exception e) {
					// Couldn't load settings
					log.info("Couldn't load settings in installer: ", e);
					if (daoFactory != null) {
						if (log.isDebugEnabled())
							log.debug("Rollback transaction to reset Hibernate.");
						daoFactory.rollbackTransaction();
					}
				}
			}
			if (settings != null && settings.getMap() != null
					&& !settings.getMap().isEmpty()) {
				databaseObjectsCreated = true;
			}
		}

		// Has a log file directory been configured?
		String log4jPropertiesFile = "log4j.properties";
		if (System.getProperty("log4j.configuration") != null) {
			log4jPropertiesFile = System.getProperty("log4j.configuration");
		}
		Properties log4j = AppUtils.loadProperties(log4jPropertiesFile,
				getServlet().getServletContext());
		String logFile = log4j.getProperty("log4j.appender.rollingfile.File");
		if (logFile != null && logFile.length() > 0
				&& !logFile.startsWith("${catalina.home}")) {
			logFileDirectoryConfigured = true;
		}

		// Put the results of the analysis in the request scope
		request.setAttribute("licenseKeyInstalled", new Boolean(
				twoWayKeyFileExists));
		request.setAttribute("twoWayKeyFileExists", new Boolean(
				twoWayKeyFileExists));
		request.setAttribute("connectedToDatabase", new Boolean(
				connectedToDatabase));
		request.setAttribute("databaseObjectsCreated", new Boolean(
				databaseObjectsCreated));
		request.setAttribute("logFileDirectoryConfigured", new Boolean(
				logFileDirectoryConfigured));
		request.setAttribute("installerLocked", new Boolean(installerLocked));
		if (log.isInfoEnabled()) {
			log.info("Installer status ...");
			log.info("licenseKeyInstalled: " + licenseKeyInstalled);
			log.info("twoWayKeyFileExists: " + twoWayKeyFileExists);
			log.info("connectedToDatabase: " + connectedToDatabase);
			log.info("databaseObjectsCreated: " + databaseObjectsCreated);
			log.info("logFileDirectoryConfigured: "
					+ logFileDirectoryConfigured);
			log.info("installerLocked: " + installerLocked);
		}

		// Prepopulate two way key file setting.
		if (twoWayKeyFile == null || twoWayKeyFile.length() == 0) {
			dynaForm.set(
					"twoWayKeyFile",
					request.getSession().getServletContext()
							.getRealPath("/WEB-INF/conf/keys/twoWay.key"));
		} else {
			dynaForm.set("twoWayKeyFile", twoWayKeyFile);
		}

		// Prepopulate URL settings
		if (request.getServerPort() == 80 && !request.getServerName().equals("localhost")) {
			dynaForm.set("customerURL", "http://" + request.getServerName()
					+ request.getContextPath());
			dynaForm.set(
					"customerSecureURL",
					"https://" + request.getServerName()
							+ request.getContextPath());
			dynaForm.set(
					"administratorURL",
					"https://" + request.getServerName()
							+ request.getContextPath() + "/administrator");
		} else {
			dynaForm.set("customerURL", "http://" + request.getServerName()
					+ ":" + request.getServerPort() + request.getContextPath());
			dynaForm.set(
					"customerSecureURL",
					"http://" + request.getServerName() + ":"
							+ request.getServerPort()
							+ request.getContextPath());
			dynaForm.set(
					"administratorURL",
					"http://" + request.getServerName() + ":"
							+ request.getServerPort()
							+ request.getContextPath() + "/administrator");
		}

		// Pre-populate log file setting.
		String catalinaHome = System.getProperty("catalina.home");
		if (!logFileDirectoryConfigured && catalinaHome != null
				&& catalinaHome.length() > 0) {
			dynaForm.set("logFileDirectory",
					catalinaHome + System.getProperty("file.separator")
							+ "logs" + System.getProperty("file.separator"));
		} else if (!logFileDirectoryConfigured) {
			dynaForm.set("logFileDirectory", request.getSession()
					.getServletContext().getRealPath("/WEB-INF/logs/"));
		} else {
			dynaForm.set("logFileDirectory",
					logFile.substring(0, logFile.lastIndexOf("/") + 1));
		}
		return mapping.findForward("success");
	}
}
