package com.softslate.commerce.installer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.businessobjects.core.SettingsBean;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.daos.core.DAOFactory;

/**
 * Do all the necessary functions required to process an installation.
 * 
 * @author David Tobey
 */
public class BasicProcessAction extends BaseAction {

	static Log log = LogFactory.getLog(BasicProcessAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		BasicInstallerForm dynaForm = (BasicInstallerForm) form;

		// Create the two way key file
		String createTwoWayKey = (String) dynaForm.get("createTwoWayKey");
		if (createTwoWayKey != null && createTwoWayKey.equals("true")) {
			CreateKeysAction createKeysAction = new CreateKeysAction();
			createKeysAction.setServlet(getServlet());
			ActionForward af2 = createKeysAction.execute(mapping, form,
					request, response);
			if (af2.getName().equals("error")) {
				return mapping.findForward("error");
			}
		}

		// Set the database settings in hibernate.properties and test the
		// connection
		String doDatabaseSettings = (String) dynaForm.get("doDatabaseSettings");
		if (doDatabaseSettings != null && doDatabaseSettings.equals("true")) {
			DBSettingsAction dbSettingsAction = new DBSettingsAction();
			dbSettingsAction.setServlet(getServlet());
			ActionForward af = dbSettingsAction.execute(mapping, form, request,
					response);
			if (af.getName().equals("error")) {
				return mapping.findForward("error");
			}
		}

		// Initialize the database with store settings, security settings and an
		// admin user.
		String doDatabaseInitialize = (String) dynaForm
				.get("doDatabaseInitialize");
		if (doDatabaseInitialize != null && doDatabaseInitialize.equals("true")) {

			// First, check if the database objects have been created
			// Presumably, to get here, we already have a valid connection
			boolean databaseObjectsCreated = false;
			DAOFactory daoFactory = (DAOFactory) servlet.getServletContext()
					.getAttribute("daoFactory");
			Settings settings = (Settings) servlet.getServletContext()
					.getAttribute("settings");
			if (settings == null || settings.getMap() == null
					|| settings.getMap().isEmpty()) {
				try {
					SettingsBean settingsBean = new SettingsBean();
					settingsBean.setDaoFactory(daoFactory);
					settingsBean.initialize();
					settings = settingsBean;
				} catch (Exception e) {
					// Couldn't load settings
					log.info("Couldn't load settings in installer: "
							+ e.toString());
					if (daoFactory != null) {
						if (log.isDebugEnabled())
							log
									.debug("Rollback transaction to reset Hibernate.");
						daoFactory.rollbackTransaction();
					}
				}
			}

			// Add a message if the objects are already there
			if (settings != null && settings.getMap() != null
					&& !settings.getMap().isEmpty()) {
				databaseObjectsCreated = true;
				ActionMessages messages = new ActionMessages();
				messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
						"installer.alreadyInitialized"));
				addMessages(request, messages);
			}

			if (!databaseObjectsCreated) {
				
				// Check here that an admin user and password were submitted
				boolean adminCredentialsError = false;
				if (!dynaForm.stringExists("administratorUserName")) {
					String message = dynaForm.getMessage("installer/installer",
							"installer.administratorUserName", null);
					dynaForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.required", message));
					adminCredentialsError = true;
				}
				if (!dynaForm.stringExists("administratorPassword")) {
					String message = dynaForm.getMessage("installer/installer",
							"installer.administratorPassword", null);
					dynaForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.required", message));
					adminCredentialsError = true;
				}
				if (adminCredentialsError) {
					addErrors(request, dynaForm.getErrors());
					return mapping.findForward("error");
				}
				
				DBInitializeAction dbInitializeAction = new DBInitializeAction();
				dbInitializeAction.setServlet(getServlet());
				ActionForward af2 = dbInitializeAction.execute(mapping, form,
						request, response);
				if (af2.getName().equals("error")) {
					return mapping.findForward("error");
				}

				StoreSettingsAction storeSettingsAction = new StoreSettingsAction();
				storeSettingsAction.setServlet(getServlet());
				ActionForward af3 = storeSettingsAction.execute(mapping, form,
						request, response);
				if (af3.getName().equals("error")) {
					return mapping.findForward("error");
				}

				SecuritySettingsAction securitySettingsAction = new SecuritySettingsAction();
				securitySettingsAction.setServlet(getServlet());
				ActionForward af4 = securitySettingsAction.execute(mapping,
						form, request, response);
				if (af4.getName().equals("error")) {
					return mapping.findForward("error");
				}
			}
		}

		// Configure the log file
		String doLogFileDirectory = (String) dynaForm.get("doLogFileDirectory");
		if (doLogFileDirectory != null && doLogFileDirectory.equals("true")) {
			ConfigureLogFileAction action = new ConfigureLogFileAction();
			action.setServlet(getServlet());
			ActionForward af5 = action.execute(mapping, form,
					request, response);
			if (af5.getName().equals("error")) {
				return mapping.findForward("error");
			}
		}

		// Always lock the installer if everything else succeeded
		LockInstallerAction lockInstallerAction = new LockInstallerAction();
		lockInstallerAction.setServlet(getServlet());
		ActionForward af5 = lockInstallerAction.execute(mapping, form, request,
				response);
		if (af5.getName().equals("error")) {
			return mapping.findForward("error");
		}

		if (log.isDebugEnabled())
			log.debug("Finished with Action.");
		return mapping.findForward("success");
	}
}
