package com.softslate.commerce.installer;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Properties;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.action.DynaActionForm;

import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.ConfigPlugIn;
import com.softslate.commerce.daos.core.DAOUtils;
import com.softslate.commerce.daos.core.DAOUtilsImpl;

/**
 * 
 * @author David Tobey
 */
public class CreateKeysAction extends BaseAction {

	static Log log = LogFactory.getLog(CreateKeysAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		ActionErrors errors = new ActionErrors();
		ActionMessages messages = new ActionMessages();
		DynaActionForm dynaForm = (DynaActionForm) form;
		boolean error = false;

		// Attempt to create the necessary key files on the server.
		DAOUtils daoUtils = new DAOUtilsImpl();
		String createTwoWayKey = (String) dynaForm.get("createTwoWayKey");
		if (createTwoWayKey != null && createTwoWayKey.equals("true")) {
			String twoWayKeyFile = (String) dynaForm.get("twoWayKeyFile");
			File keyFile = new File(twoWayKeyFile);
			if (keyFile.exists()) {
				messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
						"installer.twoWayKeyFileExists"));
			} else {
				if (log.isDebugEnabled())
					log.debug("Attempting to create key file");
				try {
					if (twoWayKeyFile.length() > 0) {
						daoUtils.generateTwoWayKey(twoWayKeyFile);
					}
					messages
							.add(ActionMessages.GLOBAL_MESSAGE,
									new ActionMessage(
											"installer.twoWayKeyFileCreated"));
				} catch (Exception e) {
					log.error(e.toString(), e);
					errors.add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("installer.keyFileError", e
									.toString()));
					error = true;
				}
			}
		}

		// Save the settings to the appSettings.properties file.
		String appSettingsPropertiesFile = "appSettings.properties";
		if (System.getProperty("appSettings.configuration") != null) {
			appSettingsPropertiesFile = System
					.getProperty("appSettings.configuration");
		}
		String appSettingsPath = getServlet().getServletContext().getRealPath(
				"/WEB-INF/classes/" + appSettingsPropertiesFile);
		if (appSettingsPath == null) {
			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"installer.warError", appSettingsPath));
			error = true;
		} else {
			try {
				Properties appSettings = null;
				if (log.isDebugEnabled())
					log.debug("Save the settings to the "
							+ appSettingsPropertiesFile + " file.");
				synchronized (getServlet().getServletContext()) {
					File appSettingsFile = new File(appSettingsPath);
					FileOutputStream fos = new FileOutputStream(appSettingsFile);
					appSettings = new Properties();
					appSettings.setProperty("twoWayKeyFile", (String) dynaForm
							.get("twoWayKeyFile"));
					appSettings
							.setProperty("initializeLegacyDAOFactory",
									(String) dynaForm
											.get("initializeLegacyDAOFactory"));
					appSettings.setProperty("forceSettingsReloadHitCount", "0");
					appSettings.setProperty("databaseTablePrefix",
							(String) dynaForm.get("databaseTablePrefix"));
					if (log.isDebugEnabled())
						log.debug("Attempting to write app settings to "
								+ appSettingsPath + " appSettings: "
								+ appSettings);
					appSettings.store(fos, "Generated by installer.");
				}
				messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
						"installer.appSettingsSaved", appSettingsPath));

				// In case we're in an environment with an src folder, save
				// the file to the src area too.
				String appSettingsPathSrc = getServlet().getServletContext()
						.getRealPath(
								"/WEB-INF/src/" + appSettingsPropertiesFile);
				File appSettingsFileSrc = new File(appSettingsPathSrc);
				if (appSettings != null && appSettingsFileSrc.exists()) {
					FileOutputStream fos3 = new FileOutputStream(
							appSettingsFileSrc);
					appSettings.store(fos3, "Generated by installer.");
					messages.add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("installer.appSettingsSaved",
									appSettingsPathSrc));
				}
			} catch (Exception fnfe) {
				log.error(fnfe.toString(), fnfe);
				errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
						"installer.fileNotFoundError", appSettingsPath));
				error = true;
			}
		}

		// Reload the appSettings object into the application scope
		ConfigPlugIn configPlugIn = new ConfigPlugIn(servlet);
		configPlugIn.initializeAppSettings();

		if (!messages.isEmpty())
			addMessages(request, messages);
		if (!errors.isEmpty())
			addErrors(request, errors);
		if (log.isDebugEnabled())
			log.debug("Finished with Action.");
		if (error) {
			return mapping.findForward("error");
		}
		return mapping.findForward("success");
	}
}
