CREATE TABLE sscCategory (
  categoryID INT(11) NOT NULL AUTO_INCREMENT,
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  keywords VARCHAR(255) DEFAULT NULL,
  description TEXT,
  isActive TINYINT(4) NOT NULL DEFAULT '1',
  isContentPage TINYINT(4) NOT NULL DEFAULT '0',
  categoryTreeID INT(11) NOT NULL DEFAULT '1',
  header TEXT,
  footer TEXT,
  smallImage VARCHAR(255) DEFAULT NULL,
  mediumImage VARCHAR(255) DEFAULT NULL,
  largeImage VARCHAR(255) DEFAULT NULL,
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  parentCategory INT(11) DEFAULT NULL,
  categoryOrder INT(11) NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.CategoryBean',
  PRIMARY KEY  (categoryID),
  UNIQUE KEY categoryCode (code)
);

CREATE INDEX IDX_sscCategory_name ON sscCategory (name);
CREATE INDEX IDX_sscCategory_parentCategory ON sscCategory (parentCategory);
CREATE INDEX IDX_sscCategory_categoryOrder ON sscCategory (categoryOrder);

CREATE TABLE sscManufacturer (
  manufacturerID INT(11) NOT NULL AUTO_INCREMENT,
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  websiteURL VARCHAR(255) DEFAULT NULL,
  description TEXT,
  isActive TINYINT(4) NOT NULL DEFAULT '1',
  header TEXT,
  footer TEXT,
  smallImage VARCHAR(255) DEFAULT NULL,
  largeImage VARCHAR(255) DEFAULT NULL,
  isDropShipper TINYINT(4) NOT NULL DEFAULT '0',
  address1 VARCHAR(255) DEFAULT NULL,
  address2 VARCHAR(255) DEFAULT NULL,
  city VARCHAR(100) DEFAULT NULL,
  state VARCHAR(100) DEFAULT NULL,
  postalCode VARCHAR(100) DEFAULT NULL,
  country VARCHAR(100) DEFAULT NULL,
  phone VARCHAR(100) DEFAULT NULL,
  email VARCHAR(100) DEFAULT NULL,
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  manufacturerOrder INT(11) NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.ManufacturerBean',
  PRIMARY KEY  (manufacturerID),
  UNIQUE KEY manufacturerCode (code)
);

CREATE INDEX IDX_sscManufacturer_name ON sscManufacturer (name);
CREATE INDEX IDX_sscManufacturer_manufacturerOrder ON sscManufacturer (manufacturerOrder);

CREATE TABLE sscProduct (
  productID INT(11) NOT NULL AUTO_INCREMENT,
  code VARCHAR(100) NOT NULL DEFAULT '',
  seoCode VARCHAR(255) DEFAULT NULL,
  name VARCHAR(255) DEFAULT NULL,
  keywords VARCHAR(255) DEFAULT NULL,
  shortDescription VARCHAR(255) DEFAULT NULL,
  description TEXT,
  isActive TINYINT(4) NOT NULL DEFAULT '1',
  unitCost DECIMAL(10,4) DEFAULT NULL,
  unitPrice DECIMAL(10,4) DEFAULT NULL,
  altPrice DECIMAL(10,4) DEFAULT NULL,
  weight DECIMAL(10,4) DEFAULT NULL,
  header TEXT,
  footer TEXT,
  isTaxed TINYINT(4) NOT NULL DEFAULT '0',
  smallImage VARCHAR(255) DEFAULT NULL,
  mediumImage VARCHAR(255) DEFAULT NULL,
  largeImage VARCHAR(255) DEFAULT NULL,
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  extra4 VARCHAR(255) DEFAULT NULL,
  extra5 VARCHAR(255) DEFAULT NULL,
  primaryCategoryID INT(11) DEFAULT NULL,
  productOrder INT(11) NOT NULL DEFAULT '0',
  created VARCHAR(100),
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.ProductBean',
  manufacturerID INT(11) DEFAULT NULL,
  PRIMARY KEY  (productID),
  UNIQUE KEY productCodeUKey (code),
  UNIQUE KEY productSEOCodeUKey (seoCode),
  KEY primaryCategoryIDIndex (primaryCategoryID),
  FOREIGN KEY (`primaryCategoryID`) REFERENCES `sscCategory` (`categoryID`) ON DELETE SET NULL,
  FOREIGN KEY (`manufacturerID`) REFERENCES `sscManufacturer` (`manufacturerID`) ON DELETE SET NULL
);

CREATE INDEX IDX_sscProduct_name ON sscProduct (name);
CREATE INDEX IDX_sscProduct_shortDescription ON sscProduct (shortDescription);
CREATE INDEX IDX_sscProduct_primaryCategoryID ON sscProduct (primaryCategoryID);
CREATE INDEX IDX_sscProduct_productOrder ON sscProduct (productOrder);

CREATE TABLE sscProductCategory (
  productCategoryID INT(11) NOT NULL AUTO_INCREMENT,
  categoryID INT(11) NOT NULL DEFAULT '0',
  productID INT(11) NOT NULL DEFAULT '0',
  productCategoryOrder INT(11) NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.ProductCategoryBean',
  PRIMARY KEY  (productCategoryID),
  KEY categoryIDIndex (categoryID),
  KEY productIDIndex (productID),
  UNIQUE KEY productCategoryUKey (productID, categoryID),
  FOREIGN KEY (`categoryID`) REFERENCES `sscCategory` (`categoryID`) ON DELETE CASCADE,
  FOREIGN KEY (`productID`) REFERENCES `sscProduct` (`productID`) ON DELETE CASCADE
);

CREATE TABLE sscAttribute (
  attributeID INT(11) NOT NULL AUTO_INCREMENT,
  attributeCode VARCHAR(100) NOT NULL DEFAULT '',
  attributeName VARCHAR(255) DEFAULT NULL,
  attributeDescription TEXT,
  attributeIsActive TINYINT(4) NOT NULL DEFAULT '1',
  attributeType VARCHAR(32) NOT NULL DEFAULT '',
  attributeUnitCost DECIMAL(10,4) DEFAULT NULL,
  attributeUnitPrice DECIMAL(10,4) DEFAULT NULL,
  attributeAltPrice DECIMAL(10,4) DEFAULT NULL,
  attributeWeight DECIMAL(10,4) DEFAULT NULL,
  attributeSmallImage VARCHAR(255) DEFAULT NULL,
  attributeLargeImage VARCHAR(255) DEFAULT NULL,
  defaultOptionID INT(11) DEFAULT NULL,
  attributeIsRequired TINYINT(4) NOT NULL DEFAULT '0',
  attributeOrder INT(11) NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.AttributeBean',
  PRIMARY KEY  (attributeID),
  UNIQUE KEY attributeCodeUKey (attributeCode)
);

CREATE INDEX IDX_sscAttribute_code ON sscAttribute (attributeCode);
CREATE INDEX IDX_sscAttribute_name ON sscAttribute (attributeName);
CREATE INDEX IDX_sscAttribute_order ON sscAttribute (attributeOrder);

CREATE TABLE sscOption (
  optionID INT(11) NOT NULL AUTO_INCREMENT,
  attributeID INT(11) NOT NULL DEFAULT '0',
  optionCode VARCHAR(100) NOT NULL DEFAULT '',
  optionName VARCHAR(255) DEFAULT NULL,
  optionDescription TEXT,
  optionIsActive TINYINT(4) NOT NULL DEFAULT '1',
  optionUnitCost DECIMAL(10,4) DEFAULT NULL,
  optionUnitPrice DECIMAL(10,4) DEFAULT NULL,
  optionAltPrice DECIMAL(10,4) DEFAULT NULL,
  optionWeight DECIMAL(10,4) DEFAULT NULL,
  optionSmallImage VARCHAR(255) DEFAULT NULL,
  optionLargeImage VARCHAR(255) DEFAULT NULL,
  optionOrder INT(11) NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.OptionBean',
  PRIMARY KEY  (optionID),
  KEY attributeIDIndex (attributeID),
  UNIQUE KEY optionAttributeUKey (optionCode, attributeID),
  FOREIGN KEY (`attributeID`) REFERENCES `sscAttribute` (`attributeID`) ON DELETE CASCADE
);

CREATE INDEX IDX_sscOption_code ON sscOption (optionCode);
CREATE INDEX IDX_sscOption_name ON sscOption (optionName);
CREATE INDEX IDX_sscOption_order ON sscOption (optionOrder);

CREATE TABLE sscProductAttribute (
  productAttributeID INT(11) NOT NULL AUTO_INCREMENT,
  productID INT(11) NOT NULL DEFAULT '0',
  attributeID INT(11) NOT NULL DEFAULT '0',
  productAttributeOrder INT NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.ProductAttributeBean',
  PRIMARY KEY  (productAttributeID),
  KEY attributeIDIndex (attributeID),
  KEY productIDIndex (productID),
  UNIQUE KEY productAttributeUKey (productID, attributeID),
  FOREIGN KEY (`attributeID`) REFERENCES `sscAttribute` (`attributeID`) ON DELETE CASCADE,
  FOREIGN KEY (`productID`) REFERENCES `sscProduct` (`productID`) ON DELETE CASCADE
);

CREATE TABLE sscSKU (
  skuID INT(11) NOT NULL AUTO_INCREMENT,
  productID INT(11) DEFAULT NULL,
  attributeID1 INT(11) DEFAULT NULL,
  optionID1 INT(11) DEFAULT NULL,
  attributeID2 INT(11) DEFAULT NULL,
  optionID2 INT(11) DEFAULT NULL,
  attributeID3 INT(11) DEFAULT NULL,
  optionID3 INT(11) DEFAULT NULL,
  attributeID4 INT(11) DEFAULT NULL,
  optionID4 INT(11) DEFAULT NULL,
  attributeID5 INT(11) DEFAULT NULL,
  optionID5 INT(11) DEFAULT NULL,
  skuNumber VARCHAR(128) DEFAULT NULL,
  skuName VARCHAR(255) DEFAULT NULL,
  skuMessage VARCHAR(255) DEFAULT NULL,
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  trackInventory TINYINT(4) NOT NULL DEFAULT '1',
  inventoryLevel INT(11) NOT NULL DEFAULT '0',
  notes VARCHAR(255) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.SKUBean',
  PRIMARY KEY  (skuID),
  UNIQUE KEY skuNumberKey (skuNumber),
  KEY productIDIndex (productID),
  FOREIGN KEY (`productID`) REFERENCES `sscProduct` (`productID`) ON DELETE CASCADE,
  UNIQUE KEY skuUKey (productID, attributeID1, optionID1, attributeID2, optionID2, 
  attributeID3, optionID3, attributeID4, optionID4, attributeID5, optionID5)
);

CREATE TABLE sscProductSetting (
  productSettingID INT(11) NOT NULL AUTO_INCREMENT,
  productID INT(11) NOT NULL,
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  description VARCHAR(255) DEFAULT NULL,
  type VARCHAR(100) DEFAULT NULL,
  valueType VARCHAR(16) NOT NULL,
  integerValue INT(11) DEFAULT NULL,
  smallValue VARCHAR(64) DEFAULT NULL,
  mediumValue VARCHAR(255) DEFAULT NULL,
  textValue TEXT,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.ProductSettingBean',
  PRIMARY KEY  (productSettingID),
  UNIQUE KEY productSettingUKey (productID, code),
  KEY productIDIndex (productID),
  FOREIGN KEY (`productID`) REFERENCES `sscProduct` (`productID`) ON DELETE CASCADE
);

CREATE TABLE sscCategorySetting (
  categorySettingID INT(11) NOT NULL AUTO_INCREMENT,
  categoryID INT(11) NOT NULL,
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  description VARCHAR(255) DEFAULT NULL,
  type VARCHAR(100) DEFAULT NULL,
  valueType VARCHAR(16) NOT NULL,
  integerValue INT(11) DEFAULT NULL,
  smallValue VARCHAR(64) DEFAULT NULL,
  mediumValue VARCHAR(255) DEFAULT NULL,
  textValue TEXT,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.CategorySettingBean',
  PRIMARY KEY  (categorySettingID),
  UNIQUE KEY categorySettingUKey (categoryID, code),
  KEY categoryIDIndex (categoryID),
  FOREIGN KEY (`categoryID`) REFERENCES `sscCategory` (`categoryID`) ON DELETE CASCADE
);

CREATE TABLE sscDiscount (
  discountID INT(11) NOT NULL AUTO_INCREMENT,
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  description TEXT,
  isActive TINYINT(4) NOT NULL DEFAULT '1',
  usersAppliesType VARCHAR(32) DEFAULT NULL,
  usersAppliesValue VARCHAR(255) DEFAULT NULL,
  itemsAppliesType VARCHAR(32) DEFAULT NULL,
  itemsAppliesValue VARCHAR(255) DEFAULT NULL,
  cartItemsType VARCHAR(32) DEFAULT NULL,
  cartItemsValue VARCHAR(255) DEFAULT NULL,
  rewardType VARCHAR(32) DEFAULT NULL,
  reward DECIMAL(10,4) NULL,
  couponCode VARCHAR(100) DEFAULT NULL,
  startDate VARCHAR(100) DEFAULT NULL,
  expirationDate VARCHAR(100) DEFAULT NULL,
  timesUsed INT(11) DEFAULT NULL,
  maxTimesUsed INT(11) DEFAULT NULL,
  isTimesPerCustomer TINYINT(4) NOT NULL DEFAULT '1',
  orderItemID INT(11) DEFAULT NULL,
  discountOrder INT(11) NOT NULL DEFAULT '0',
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.DiscountBean',
  PRIMARY KEY  (discountID),
  UNIQUE KEY discountCodeUKey (code)
);

CREATE INDEX IDX_sscDiscount_name ON sscDiscount (name);
CREATE INDEX IDX_sscDiscount_discountOrder ON sscDiscount (discountOrder);

CREATE TABLE sscSKUDiscount (
  skuDiscountID INT(11) NOT NULL AUTO_INCREMENT,
  skuID INT(11) NOT NULL DEFAULT '0',
  discountID INT(11) NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.SKUDiscountBean',
  PRIMARY KEY  (skuDiscountID),
  UNIQUE KEY discountCodeUKey (skuID, discountID),
  KEY skuIDIndex (skuID),
  FOREIGN KEY (`skuID`) REFERENCES `sscSKU` (`skuID`) ON DELETE CASCADE,
  KEY discountIDIndex (discountID),
  FOREIGN KEY (`discountID`) REFERENCES `sscDiscount` (`discountID`) ON DELETE CASCADE
);


CREATE TABLE sscDiscountRange (
  discountRangeID INT(11) NOT NULL AUTO_INCREMENT,
  discountID INT(11) NOT NULL,
  floor DECIMAL(10,4) NOT NULL DEFAULT '0',
  ceiling DECIMAL(10,4) NOT NULL DEFAULT '0',
  rewardType VARCHAR(32) DEFAULT NULL,
  reward DECIMAL(10,4) NOT NULL DEFAULT '0',
  isActive TINYINT(4) NOT NULL DEFAULT '1',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.DiscountRangeBean',
  PRIMARY KEY  (discountRangeID),
  KEY discountIDIndex (discountID),
  FOREIGN KEY (`discountID`) REFERENCES `sscDiscount` (`discountID`) ON DELETE CASCADE
);

CREATE INDEX IDX_sscDiscountRange_reward ON sscDiscountRange (reward);

CREATE TABLE sscRelatedProduct (
  relatedProductID INT(11) NOT NULL AUTO_INCREMENT,
  parentProductID INT(11) NOT NULL DEFAULT '0',
  productID INT(11) NOT NULL DEFAULT '0',
  relatedType VARCHAR(32) DEFAULT NULL,
  relatedProductOrder INT(11) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.RelatedProductBean',
  PRIMARY KEY  (relatedProductID),
  UNIQUE KEY relatedProductUKey (parentProductID, productID),
  KEY parentProductIDIndex (parentProductID),
  FOREIGN KEY (`parentProductID`) REFERENCES `sscProduct` (`productID`) ON DELETE CASCADE,
  KEY productIDIndex (productID),
  FOREIGN KEY (`productID`) REFERENCES `sscProduct` (`productID`)
);

CREATE TABLE sscProductReview (
  productReviewID INT(11) NOT NULL AUTO_INCREMENT,
  productID INT(11) NOT NULL,
  status INT(11) DEFAULT 0,
  class VARCHAR(255) NOT NULL DEFAULT 'com.softslate.commerce.businessobjects.product.ProductReviewBean',
  nickname VARCHAR(255),
  firstName VARCHAR(255),
  lastName VARCHAR(255),
  email VARCHAR(255),
  age VARCHAR(255),
  city VARCHAR(255),
  state VARCHAR(255),
  review text,
  rating INT(11),
  created VARCHAR(255),
  imagePath VARCHAR(255),
  ipaddress VARCHAR(255),
  country VARCHAR(255),
  customerID INT(11),
  PRIMARY KEY  (productReviewID),
  FOREIGN KEY (`productID`) REFERENCES `sscProduct` (`productID`) ON DELETE CASCADE
);

CREATE INDEX IDX_sscProductReview_rating ON sscProductReview (rating);
CREATE INDEX IDX_sscProductReview_nickname ON sscProductReview (nickname);

