package com.softslate.commerce.businessobjects.core;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.daos.core.DAOFactory;

public class TestSupport {

	static Log log = LogFactory.getLog(TestSupport.class);

	public BusinessObjectFactory mockedBOF = null;
	public DAOFactory mockedDaoFactory = null;

	public void stubUserObject() throws Exception {
		User user = (User) mockedBOF.createObject("userImplementer");
		Order o = (Order) mockedBOF.createObject("orderImplementer");
		Customer c = (Customer) mockedBOF.createObject("customerImplementer");
		when(user.getOrder()).thenReturn(o);
		when(user.getCustomer()).thenReturn(c);
		when(user.getLocale()).thenReturn(Locale.getDefault());
	}

	public void stubBusinessProcessor(BusinessProcessor bp) throws Exception {
		User user = (User) mockedBOF.createObject("userImplementer");
		Settings settings = (Settings) mockedBOF
				.createObject("settingsImplementer");
		when(bp.getUser()).thenReturn(user);
		when(bp.getSettings()).thenReturn(settings);
		when(bp.getBusinessObjectFactory()).thenReturn(mockedBOF);
		when(bp.getDaoFactory()).thenReturn(mockedDaoFactory);
	}

	public void stubUnmockedBusinessProcessor(BusinessProcessor bp) throws Exception {
		User user = (User) mockedBOF.createObject("userImplementer");
		Settings settings = (Settings) mockedBOF
				.createObject("settingsImplementer");
		bp.setUser(user);
		bp.setSettings(settings);
		bp.setBusinessObjectFactory(mockedBOF);
		bp.setDaoFactory(mockedDaoFactory);
	}

	public void mockEverything() {
		mockedBOF = mock(BusinessObjectFactory.class);
		mockedDaoFactory = mock(DAOFactory.class);
		Properties defaults = null;
		Properties appComponentsDefaults = AppUtils.loadProperties(
				"/appComponents.properties", defaults);
		Properties customComponents = AppUtils.loadProperties(
				"/appComponents-custom.properties", appComponentsDefaults);
		Map components = new HashMap();
		components.putAll(appComponentsDefaults);
		components.putAll(customComponents);
		Collection keySet = components.keySet();
		Iterator i = keySet.iterator();
		while (i.hasNext()) {
			String key = (String) i.next();
			String[] classNames = ((String) components.get(key)).split(",");
			for (int j = 0; j < classNames.length; j++) {
				String className = classNames[j].trim();
				if (!className.endsWith(".jsp") && className.length() > 0) {
					if (key.endsWith("DAOImplementer")) {
						try {
							Class classDefinition = Class.forName(className);
							Object obj = mock(classDefinition);
							when(mockedDaoFactory.createDAO(key)).thenReturn(
									obj);
							when(
									mockedDaoFactory
											.createDAOFromClassName(className))
									.thenReturn(obj);
						} catch (Exception e) {
						}
					} else if (key.endsWith("UtilsImplementer")) {
						try {
							Class classDefinition = Class.forName(className);
							Object obj = mock(classDefinition);
							when(mockedBOF.createUtilsObject(key)).thenReturn(
									obj);
							when(mockedBOF.createObjectFromClassName(className))
.thenReturn((BusinessObject) obj);
						} catch (Exception e) {
						}
					} else {
						try {
							Class classDefinition = Class.forName(className);
							BusinessObject obj = mock(classDefinition);
							when(mockedBOF.createObject(key)).thenReturn(obj);
							when(mockedBOF.createObjectFromClassName(className))
									.thenReturn(obj);
						} catch (Exception e) {
						}
					}
				}
			}
		}

	}
}
